@@ -5,7 +5,9 @@
  * @author Attila Kovacs
  */
 
-#define _POSIX_C_SOURCE 199309L   ///< for clock_gettime()
+#if __STDC_VERSION__ < 201112L
+#  define _POSIX_C_SOURCE 199309      ///< struct timespec
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -20,6 +22,11 @@
 #define  POLAR_DX         230.0     ///< [mas] Earth polar offset x, e.g. from IERS Bulletin A.
 #define  POLAR_DY         -62.0     ///< [mas] Earth polar offset y, e.g. from IERS Bulletin A.
 
+
+static void timestamp(novas_timespec *t) {
+  novas_set_current_time(LEAP_SECONDS, DUT1, t);
+}
+
 static void calc_pos(const cat_entry *star, const novas_frame *frame) {
   object source = NOVAS_OBJECT_INIT;
   sky_pos apparent = SKY_POS_INIT;
@@ -53,12 +60,13 @@ int main(int argc, const char *argv[]) {
 
 
   // Intermediate variables we'll use -------------------------------------->
-  struct timespec unix_time, end;   // Standard precision UNIX time structure
+  novas_timespec start, end;        // timestamps for execution time
 
 
   // Other variables we need ----------------------------------------------->
-  int i, N = 300000;
+  int i, N = 300000, N2 = N  / 10, N3 = N / 30;
 
+  novas_debug(1);
 
   if(argc > 1) N = (int) strtol(argv[1], NULL, 10);
 
@@ -90,11 +98,8 @@ int main(int argc, const char *argv[]) {
   // -------------------------------------------------------------------------
   // Set the astrometric time of observation...
 
-  // Get the current system time, with up to nanosecond resolution...
-  clock_gettime(CLOCK_REALTIME, &unix_time);
-
-  // Set the time of observation to the precise UTC-based UNIX time
-  if(novas_set_unix_time(unix_time.tv_sec, unix_time.tv_nsec, LEAP_SECONDS, DUT1, &obs_time) != 0) {
+  // Set the time of observation to the current UTC-based UNIX time
+  if(novas_set_current_time(LEAP_SECONDS, DUT1, &obs_time) != 0) {
     fprintf(stderr, "ERROR! failed to set time of observation.\n");
     return 1;
   }
@@ -140,39 +145,39 @@ int main(int argc, const char *argv[]) {
 
   // -------------------------------------------------------------------------
   // Benchmark reduced accuracy, same frame
-  clock_gettime(CLOCK_REALTIME, &unix_time);
+  timestamp(&start);
   for(i = 0; i < N; i++) calc_pos(&stars[i], &obs_frame);
-  clock_gettime(CLOCK_REALTIME, &end);
+  timestamp(&end);
   printf(" - novas_sky_pos(), same frame, red. acc.:        %12.1f positions/sec\n",
-          N / (end.tv_sec - unix_time.tv_sec + 1e-9 * (end.tv_nsec - unix_time.tv_nsec)));
+          N / novas_diff_time(&end, &start));
 
   // -------------------------------------------------------------------------
   // Benchmark full accuracy, same frame
   obs_frame.accuracy = NOVAS_FULL_ACCURACY;
-  clock_gettime(CLOCK_REALTIME, &unix_time);
+  timestamp(&start);
   for(i = 0; i < N; i++) calc_pos(&stars[i], &obs_frame);
-  clock_gettime(CLOCK_REALTIME, &end);
-  printf(" - novas_sky_pos, same frame, full acc.:          %12.1f positions/sec\n",
-          N / (end.tv_sec - unix_time.tv_sec + 1e-9 * (end.tv_nsec - unix_time.tv_nsec)));
+  timestamp(&end);
+  printf(" - novas_sky_pos(), same frame, full acc.:        %12.1f positions/sec\n",
+          N / novas_diff_time(&end, &start));
 
 
 
   // -------------------------------------------------------------------------
   // Benchmark place() reduced accuracy, same frame
-  clock_gettime(CLOCK_REALTIME, &unix_time);
+  timestamp(&start);
   for(i = 0; i < N; i++) calc_place(&stars[i], &obs_frame);
-  clock_gettime(CLOCK_REALTIME, &end);
+  timestamp(&end);
   printf(" - place(), same frame, red. acc.:                %12.1f positions/sec\n",
-          N / (end.tv_sec - unix_time.tv_sec + 1e-9 * (end.tv_nsec - unix_time.tv_nsec)));
+          N / novas_diff_time(&end, &start));
 
   // -------------------------------------------------------------------------
   // Benchmark place() full accuracy, same frame
   obs_frame.accuracy = NOVAS_FULL_ACCURACY;
-  clock_gettime(CLOCK_REALTIME, &unix_time);
+  timestamp(&start);
   for(i = 0; i < N; i++) calc_place(&stars[i], &obs_frame);
-  clock_gettime(CLOCK_REALTIME, &end);
+  timestamp(&end);
   printf(" - place(), same frame, full acc.:                %12.1f positions/sec\n",
-          N / (end.tv_sec - unix_time.tv_sec + 1e-9 * (end.tv_nsec - unix_time.tv_nsec)));
+          N / novas_diff_time(&end, &start));
 
 
   // individual frames are expected to be significantly slower, so
@@ -181,53 +186,52 @@ int main(int argc, const char *argv[]) {
 
   // -------------------------------------------------------------------------
   // Benchmark reduced accuracy, individual fames
-  clock_gettime(CLOCK_REALTIME, &unix_time);
-  for(i = 0; i < N; i++) {
-    novas_set_unix_time(unix_time.tv_sec + i, unix_time.tv_nsec, LEAP_SECONDS, DUT1, &obs_time);
+  timestamp(&start);
+  for(i = 0; i < N2; i++) {
+    novas_set_time(NOVAS_TT, novas_get_time(&start, NOVAS_TT) + i, LEAP_SECONDS, DUT1, &obs_time);
     novas_make_frame(NOVAS_REDUCED_ACCURACY, &obs, &obs_time, POLAR_DX, POLAR_DY, &obs_frame);
     calc_pos(&stars[i], &obs_frame);
   }
-  clock_gettime(CLOCK_REALTIME, &end);
+  timestamp(&end);
   printf(" - novas_sky_pos, individual, red. acc.:          %12.1f positions/sec\n",
-          N / (end.tv_sec - unix_time.tv_sec + 1e-9 * (end.tv_nsec - unix_time.tv_nsec)));
+          N2 / novas_diff_time(&end, &start));
 
   // -------------------------------------------------------------------------
   // Benchmark full accuracy, individual frames
-  clock_gettime(CLOCK_REALTIME, &unix_time);
-  for(i = 0; i < N; i++) {
-    novas_set_unix_time(unix_time.tv_sec + i, unix_time.tv_nsec, LEAP_SECONDS, DUT1, &obs_time);
+  timestamp(&start);
+  for(i = 0; i < N3; i++) {
+    novas_set_time(NOVAS_TT, novas_get_time(&start, NOVAS_TT) + i, LEAP_SECONDS, DUT1, &obs_time);
     novas_make_frame(NOVAS_FULL_ACCURACY, &obs, &obs_time, POLAR_DX, POLAR_DY, &obs_frame);
     calc_pos(&stars[i], &obs_frame);
   }
-  clock_gettime(CLOCK_REALTIME, &end);
+  timestamp(&end);
   printf(" - novas_sky_pos, individual, full acc.:          %12.1f positions/sec\n",
-          N / (end.tv_sec - unix_time.tv_sec + 1e-9 * (end.tv_nsec - unix_time.tv_nsec)));
-
+          N3 / novas_diff_time(&end, &start));
 
 
   // -------------------------------------------------------------------------
   // Benchmark place() reduced accuracy, individual frames
   obs_frame.accuracy = NOVAS_REDUCED_ACCURACY;
-  clock_gettime(CLOCK_REALTIME, &unix_time);
-  for(i = 0; i < N; i++) {
-    obs_frame.time.ijd_tt += (i % 2) ? 1 : -1;  // alternate dates.
+  timestamp(&start);
+  for(i = 0; i < N2; i++) {
+    novas_set_time(NOVAS_TT, novas_get_time(&start, NOVAS_TT) + ((i % 2) ? 1 : -1), LEAP_SECONDS, DUT1, &obs_frame.time);
     calc_place(&stars[i], &obs_frame);
   }
-  clock_gettime(CLOCK_REALTIME, &end);
+  timestamp(&end);
   printf(" - place(), individual, red. acc.:                %12.1f positions/sec\n",
-          N / (end.tv_sec - unix_time.tv_sec + 1e-9 * (end.tv_nsec - unix_time.tv_nsec)));
+          N2 / novas_diff_time(&end, &start));
 
   // -------------------------------------------------------------------------
   // Benchmark place full accuracy, individual frames
   obs_frame.accuracy = NOVAS_FULL_ACCURACY;
-  clock_gettime(CLOCK_REALTIME, &unix_time);
-  for(i = 0; i < N; i++) {
-    obs_frame.time.ijd_tt += (i % 2) ? 1 : -1;
+  timestamp(&start);
+  for(i = 0; i < N3; i++) {
+    novas_set_time(NOVAS_TT, novas_get_time(&start, NOVAS_TT) + ((i % 2) ? 1 : -1), LEAP_SECONDS, DUT1, &obs_frame.time);
     calc_place(&stars[i], &obs_frame);
   }
-  clock_gettime(CLOCK_REALTIME, &end);
+  timestamp(&end);
   printf(" - place(), individual, full acc.:                %12.1f positions/sec\n",
-          N / (end.tv_sec - unix_time.tv_sec + 1e-9 * (end.tv_nsec - unix_time.tv_nsec)));
+          N3 / novas_diff_time(&end, &start));
 
 
   return 0;
