@@ -45,9 +45,6 @@ int main() {
   // Calculated quantities ------------------------------------------------->
   double az, el;                    // calculated azimuth and elevation at observing site
 
-  // Intermediate variables we'll use -------------------------------------->
-  struct timespec unix_time;        // Standard precision UNIX time structure
-
 
   // We'll print debugging messages and error traces...
   novas_debug(NOVAS_DEBUG_ON);
@@ -64,15 +61,25 @@ int main() {
   // Solar-System Barycenter (SSB). But you can convert LSR-based velocities to
   // the required SSB-based radial velocities using novas_lsr_to_ssb_vel() if needed.
 
-  // Convert string coordinates to hours/degrees...
-  double ra0 = novas_str_hours("16h26m20.1918s");
-  double dec0 = novas_str_degrees("-26d19m23.138s");
-
-  if(make_cat_entry("Antares", "FK4", 1, ra0, dec0, -12.11, -23.30, 5.89, -3.4, &star) != 0) {
+  // E.g. initialize with string coordinates to hours/degrees...
+  if(novas_init_cat_entry(&star, "Antares", novas_str_hours("16h26m20.1918s"), novas_str_degrees("-26d19m23.138s")) != 0) {
     fprintf(stderr, "ERROR! defining cat_entry.\n");
     return 1;
   }
 
+  // Optionally, we might store the catalog information: catalog ID (5-chars max) and number.
+  novas_set_catalog(&star, "HIP", 80763);
+
+  // Set the proper motion
+  novas_set_proper_motion(&star, -12.11, -23.30);
+
+  // Set the parallax. Alternatively we could set distance with `novas_set_distance()` instead.
+  novas_set_parallax(&star, 5.89);
+
+  // Set radial velocity. Or we could set LSR velocity with 'novas_set_lsr_vel()' instead
+  novas_set_ssb_vel(&star, -3.4);
+
+
   // -------------------------------------------------------------------------
   // Convert to ICRS coordinates and wrap in a generic object structure.
   if(make_cat_object_sys(&star, "B1950", &source) != 0) {
@@ -86,9 +93,9 @@ int main() {
   // or Sun orbit, at the geocenter or at the Solary-system barycenter...)
 
   // Specify the location we are observing from
-  // 50.7374 deg N, 7.0982 deg E, 60m elevation
-  // (We'll ignore the local weather parameters here, but you can set those too.)
-  if(make_observer_on_surface(50.7374, 7.0982, 60.0, 0.0, 0.0, &obs) != 0) {
+  // 50.7374 deg N, 7.0982 deg E, 60m elevation (GPS / WGS84)
+  // (You can set local weather parameters after...)
+  if(make_gps_observer(50.7374, 7.0982, 60.0, &obs) != 0) {
     fprintf(stderr, "ERROR! defining Earth-based observer location.\n");
     return 1;
   }
@@ -97,11 +104,8 @@ int main() {
   // -------------------------------------------------------------------------
   // Set the astrometric time of observation...
 
-  // Get the current system time, with up to nanosecond resolution...
-  clock_gettime(CLOCK_REALTIME, &unix_time);
-
-  // Set the time of observation to the precise UTC-based UNIX time
-  if(novas_set_unix_time(unix_time.tv_sec, unix_time.tv_nsec, LEAP_SECONDS, DUT1, &obs_time) != 0) {
+  // Set the time of observation to the current UTC-based UNIX time
+  if(novas_set_current_time(LEAP_SECONDS, DUT1, &obs_time) != 0) {
     fprintf(stderr, "ERROR! failed to set time of observation.\n");
     return 1;
   }
