@@ -1,27 +1,86 @@
 /**
  * @file
  *
+ *  Function relating to the use of Keplerian orbital elements. For example, the IAU Minor Planet
+ *  Center publishes up-to-date Keplerial orbital elements for all asteroids, comets, and
+ *  Near-Earth Objects (NEOs) regularly. On short timescales these can provide accurate positions
+ *  for such objects, that are as good as, or comparable to, the ephemeris data provided by NASA
+ *  JPL.
+ *
+ *  For newly discovered objects, the MPC orbital elements may be the only source, or the most
+ *  accurate source, of position information.
+ *
+ *  To use Keplerian orbital elements with __SuperNOVAS__, simply define the orbital parameters
+ *  in a `novas_orbital` structure. By default heliocentric orbits are assumed, but you may also
+ *  define orbitals around other bodies, such as a major planet, by defining the body (or
+ *  barycenter) that is at the center of the orbital, and the orientation of the orbital system
+ *  (w.r.t. the ecliptic or equator of date) with `novas_set_orbsys_pole()`. E.g.:
+ *
+ *  ```c
+ *    novas_orbital orb = NOVAS_ORBIT_INIT;
+ *
+ *    orb.a = ...    // [AU] semi-major axis
+ *    ...            // populate the rest of the orbital parameters.
+ *
+ *    // Set Jupiter as the orbital center
+ *    orb.system.center = NOVAS_JUPITER_INIT;
+ *
+ *    // Set the orientation by defining the RA/Dec of the orbital pole, say in ICRS
+ *    novas_set_orbsys_pole(NOVAS_ICRS, ra_pole, dec_pole, &orb.system);
+ *  ```
+ *
+ *  Once the orbital is defined, you can use it to define a generic @ref object via
+ *  `make_orbital_object()` to enable the the same astrometric calculations as for ephemeris
+ *  sources, e.g.:
+ *
+ *  ```c
+ *    object source;
+ *
+ *    // Let's say the orbital is that for Callisto, so we set the source name
+ *    // and NAIF ID number (504) accordingly.
+ *    make_orbital_object("Callisto", 504, &orb, &source);
+ *  ```
+ *
+ *  Or, you can calculate geometric orbital positions and velocities directly via
+ *  `novas_orbit_posvel()`, while `novas_orbit_native_posvel()` will do the same, but in
+ *  the native coordinate system in which the orbital is defined.
+ *
+ *
  * @date Created  on Mar 6, 2025
  * @author Attila Kovacs
+ * @since 1.2
  *
- *  Function relating to the use of orbital elements
+ * @sa novas_orbital, make_orbital_object()
+ * @sa ephemeris.c
  */
 
 #include <errno.h>
 
 /// \cond PRIVATE
 #define __NOVAS_INTERNAL_API__    ///< Use definitions meant for internal use by SuperNOVAS only
-#include "novas.h"
 /// \endcond
 
+#include "novas.h"
+
+/// \cond PRIVATE
+#  define EPREC               1e-12     ///< Required precision for eccentric anomaly in orbital calculation
+/// \endcond
+
+#if __cplusplus
+#  ifdef NOVAS_NAMESPACE
+namespace novas {
+#  endif
+#endif
 
 /**
- * Change xzy vectors to the new polar orientation. &theta, &phi define the orientation of the input pole in the output system.
+ * Change _xyz_ vectors to the new polar orientation. &theta, &phi define the orientation of the
+ * input pole in the output system.
  *
  * @param in        input 3-vector in the original system (pole = z)
  * @param theta     [deg] polar angle of original pole in the new system
  * @param phi       [deg] azimuthal angle of original pole in the new system
- * @param[out] out  output 3-vector in the new (rotated) system. It may be the same vector as the input.
+ * @param[out] out  output 3-vector in the new (rotated) system. It may be the same vector as the
+ *                  input.
  * @return          0
  *
  */
@@ -56,8 +115,8 @@ static int change_pole(const double *in, double theta, double phi, double *out)
  *                  MOD, or CIRS). It must be an inertial celestial system, i.e. it cannot be a
  *                  reference system which co-rotates with Earth (like ITRS to TIRS).
  * @param[in, out] vec  vector to change to GCRS.
- * @return          0 if successful, or else -1 (errno set to EINVAL) if the 'sys'
- *                  argument is invalid or unsupported.
+ * @return          0 if successful, or else -1 (errno set to EINVAL) if the 'sys' argument is
+ *                  invalid or unsupported.
  *
  * @author Attila Kovacs
  * @since 1.2
@@ -182,9 +241,9 @@ static int novas_orbital_plane_pos(double M, double e, double *restrict E, doubl
 }
 
 /**
- * Calculates a rectangular equatorial position and velocity vector for the given orbital elements for the
- * specified time of observation, in the native coordinate system in which the orbital is defined (e.g. ecliptic
- * for heliocentric orbitals).
+ * Calculates a rectangular equatorial position and velocity vector for the given orbital elements
+ * for the specified time of observation, in the native coordinate system in which the orbital is
+ * defined (e.g. ecliptic for heliocentric orbitals).
  *
  * REFERENCES:
  * <ol>
@@ -197,20 +256,18 @@ static int novas_orbital_plane_pos(double M, double e, double *restrict E, doubl
  *
  * @param jd_tdb    [day] Barycentric Dynamic Time (TDB) based Julian date
  * @param orbit     Orbital parameters
- * @param[out] pos  [AU] Output ICRS equatorial position vector around orbital center, or NULL if not required.
- * @param[out] vel  [AU/day] Output ICRS velocity vector rel. to orbital center, in the native system of the orbital,
- *                  or NULL if not required. 0 if successful, or else -1 if the orbital parameters are NULL,
- *                  or if the position and velocity output vectors are the same or the orbital
- *                  system is ill defined (errno set to EINVAL), or if the calculation did not converge (errno set to
- *                  ECANCELED), or
- *
- * @sa ephemeris()
- * @sa novas_geom_posvel()
- * @sa place()
- * @sa make_orbital_object()
+ * @param[out] pos  [AU] Output ICRS equatorial position vector around orbital center, or NULL if
+ *                  not required.
+ * @param[out] vel  [AU/day] Output ICRS velocity vector rel. to orbital center, in the native
+ *                  system of the orbital, or NULL if not required. 0 if successful, or else -1 if
+ *                  the orbital parameters are NULL, or if the position and velocity output
+ *                  vectors are the same or the orbital system is ill defined (errno set to
+ *                  EINVAL), or if the calculation did not converge (errno set to ECANCELED).
  *
  * @author Attila Kovacs
  * @since 1.4
+ *
+ * @sa novas_geom_posvel(), ephemeris(), make_orbital_object()
  */
 int novas_orbit_native_posvel(double jd_tdb, const novas_orbital *restrict orbit, double *restrict pos, double *restrict vel) {
   static const char *fn = "novas_orbit_native_posvel";
@@ -282,8 +339,8 @@ int novas_orbit_native_posvel(double jd_tdb, const novas_orbital *restrict orbit
 }
 
 /**
- * Calculates a rectangular equatorial position and velocity vector for the given orbital elements for the
- * specified time of observation.
+ * Calculates a rectangular equatorial position and velocity vector for the given orbital elements
+ * for the specified time of observation.
  *
  * REFERENCES:
  * <ol>
@@ -297,20 +354,19 @@ int novas_orbit_native_posvel(double jd_tdb, const novas_orbital *restrict orbit
  * @param jd_tdb    [day] Barycentric Dynamic Time (TDB) based Julian date
  * @param orbit     Orbital parameters
  * @param accuracy  NOVAS_FULL_ACCURACY (0) or NOVAS_REDUCED_ACCURACY (1).
- * @param[out] pos  [AU] Output ICRS equatorial position vector around orbital center, or NULL if not required.
- * @param[out] vel  [AU/day] Output ICRS equatorial velocity vector rel. to orbital center, or NULL if not required.
+ * @param[out] pos  [AU] Output ICRS equatorial position vector around orbital center, or NULL if
+ *                  not required.
+ * @param[out] vel  [AU/day] Output ICRS equatorial velocity vector rel. to orbital center, or
+ *                  NULL if not required.
  * @return          0 if successful, or else -1 if the orbital parameters are NULL,
  *                  or if the position and velocity output vectors are the same or the orbital
- *                  system is ill defined (errno set to EINVAL), or if the calculation did not converge (errno set to
- *                  ECANCELED), or
- *
- * @sa ephemeris()
- * @sa novas_geom_posvel()
- * @sa place()
- * @sa make_orbital_object()
+ *                  system is ill defined (errno set to EINVAL), or if the calculation did not
+ *                  converge (errno set to ECANCELED).
  *
  * @author Attila Kovacs
  * @since 1.2
+ *
+ * @sa novas_geom_posvel(), ephemeris(), make_orbital_object()
  */
 int novas_orbit_posvel(double jd_tdb, const novas_orbital *restrict orbit, enum novas_accuracy accuracy,
         double *restrict pos, double *restrict vel) {
@@ -374,4 +430,8 @@ int novas_set_orbsys_pole(enum novas_reference_system type, double ra, double de
   return 0;
 }
 
-
+#if __cplusplus
+#  ifdef NOVAS_NAMESPACE
+} // namespace novas
+#  endif
+#endif
