@@ -1,12 +1,18 @@
 /**
  * @file
  *
- * @date Created  on Apr 27, 2025
- * @author Attila Kovacs
- *
  *  Calculate approximate positions and velocities for the major planets, Sun, Moon, Earth-Moon
  *  Barycenter (EMB), and the Solar-system Barycenter (SSB), mainly by using Keplerian orbital
  *  elements.
+ *
+ *  For more precise planet positions, you should use SuperNOVAS with a planetary ephemeris
+ *  provider, as as the CALCEPH or CSPICE plugin interface modules, or custom ephemeris provider
+ *  functions.
+ *
+ * @date Created  on Apr 27, 2025
+ * @author Attila Kovacs
+ *
+ * @sa orbital.c, solsys-calceph.c, solsys-cspice.c, ephemeris.c
  */
 
 #include <math.h>
@@ -15,9 +21,56 @@
 
 /// \cond PRIVATE
 #define __NOVAS_INTERNAL_API__    ///< Use definitions meant for internal use by SuperNOVAS only
-#include "novas.h"
 /// \endcond
 
+#include "novas.h"
+
+#if __cplusplus
+#  ifdef NOVAS_NAMESPACE
+namespace novas {
+#  endif
+#endif
+
+/**
+ * Returns the planetary longitude, for Mercury through Neptune, w.r.t. mean dynamical ecliptic
+ * and equinox of J2000, with high order terms omitted (Simon et al. 1994, 5.8.1-5.8.8).
+ *
+ * REFERENCES:
+ * <ol>
+ * <li>IERS Conventions Chapter 5, Eq. 5.44.</li>
+ * </ol>
+ *
+ * @param t       [cy] Julian centuries since J2000
+ * @param planet  Novas planet id, e.g. NOVAS_MARS.
+ * @return        [rad] The approximate longitude of the planet in radians [-&pi;:&pi;],
+ *                or NAN if the `planet` id is out of range.
+ *
+ * @sa accum_prec(), nutation_angles(), NOVAS_JD_J2000
+ *
+ * @since 1.0
+ * @author Attila Kovacs
+ */
+double planet_lon(double t, enum novas_planet planet) {
+  static const double c[9][2] = {
+          { 0.0, 0.0 }, //
+          { 4.402608842461, 2608.790314157421 },  // Mercury
+          { 3.176146696956, 1021.328554621099 },  // Venus
+          { 1.753470459496,  628.307584999142 },  // Earth
+          { 6.203476112911,  334.061242669982 },  // Mars
+          { 0.599547105074,   52.969096264064 },  // Jupiter
+          { 0.874016284019,   21.329910496032 },  // Saturn
+          { 5.481293871537,    7.478159856729 },  // Uranus
+          { 5.311886286677,    3.813303563778 }   // Neptune
+  };
+
+  if(planet < NOVAS_MERCURY || planet > NOVAS_NEPTUNE) {
+    novas_set_errno(EINVAL, "planet_lon", "invalid planet number: %d", planet);
+    return NAN;
+  }
+
+  return remainder(c[planet][0] + c[planet][1] * t, TWOPI);
+}
+
 /**
  * Get approximate current heliocentric orbital elements for the major planets. These orbital
  * elements are not suitable for precise position velocity calculations, but they may be
@@ -53,10 +106,8 @@
  * @since 1.4
  * @author Attila Kovacs
  *
- * @sa novas_make_moon_orbit()
- * @sa novas_approx_sky_pos()
- * @sa novas_approx_heliocentric()
- * @sa make_orbital_object()
+ * @sa novas_make_moon_orbit(), novas_approx_sky_pos(), novas_approx_heliocentric(),
+ *     make_orbital_object()
  */
 int novas_make_planet_orbit(enum novas_planet id, double jd_tdb, novas_orbital *restrict orbit) {
   static const char *fn = "novas_make_planet_orbit";
@@ -224,8 +275,8 @@ int novas_make_planet_orbit(enum novas_planet id, double jd_tdb, novas_orbital *
 }
 
 /**
- * Gets the current orbital elements for the Moon relative to the geocenter for the specified
- * epoch of observation.
+ * Gets the current orbital elements for the Moon relative to the geocenter for the specified epoch
+ * of observation.
  *
  * REFERENCES:
  * <ol>
@@ -241,8 +292,7 @@ int novas_make_planet_orbit(enum novas_planet id, double jd_tdb, novas_orbital *
  * @since 1.4
  * @author Attila Kovacs
  *
- * @sa novas_make_planet_orbit()
- * @sa make_orbital_object()
+ * @sa novas_make_planet_orbit(), make_orbital_object()
  */
 int novas_make_moon_orbit(double jd_tdb, novas_orbital *restrict orbit) {
   novas_orbital def = NOVAS_ORBIT_INIT;
@@ -329,8 +379,9 @@ int novas_make_moon_orbit(double jd_tdb, novas_orbital *restrict orbit) {
  *
  * NOTES:
  * <ol>
- *  <li>The Sun's position w.r.t. the Solar-system Barycenter is calculated using `earth_sun_calc()`.
- *  All other orbitals are also referenced to the Sun's position calculated that way.</li>
+ *  <li>The Sun's position w.r.t. the Solar-system Barycenter is calculated using
+ *  `earth_sun_calc()`. All other orbitals are also referenced to the Sun's position calculated
+ *  that way.</li>
  * </ol>
  *
  * REFERENCES:
@@ -358,11 +409,8 @@ int novas_make_moon_orbit(double jd_tdb, novas_orbital *restrict orbit) {
  * @since 1.4
  * @author Attila Kovacs
  *
- * @sa novas_approx_sky_pos()
- * @sa earth_sun_calc()
- * @sa novas_geom_posvel()
- * @sa novas_use_calceph()
- * @sa novas_use_cspice()
+ * @sa novas_approx_sky_pos(), earth_sun_calc(), novas_geom_posvel()
+ * @sa novas_use_calceph(), novas_use_cspice()
  */
 int novas_approx_heliocentric(enum novas_planet id, double jd_tdb, double *restrict pos, double *restrict vel) {
   static const char *fn = "novas_approx_heliocentric";
@@ -428,8 +476,8 @@ int novas_approx_heliocentric(enum novas_planet id, double jd_tdb, double *restr
  * the CALCEPH or CSPICE plugins) and `novas_sky_pos()` instead.
  *
  * While this function is generally similar to creating an orbital object with an orbit
- * initialized with `novas_make_planet_orbit()` or `novas_make_moon_orbit()`, and then calling `novas_sky_pos()`,
- * there are a few important differences to note:
+ * initialized with `novas_make_planet_orbit()` or `novas_make_moon_orbit()`, and then calling
+ * `novas_sky_pos()`, there are a few important differences to note:
  *
  * <ol>
  *  <li>This function calculates Earth and Moon positions about the Keplerian orbital position
@@ -464,9 +512,8 @@ int novas_approx_heliocentric(enum novas_planet id, double jd_tdb, double *restr
  * @since 1.4
  * @author Attila Kovacs
  *
- * @sa novas_sky_pos()
- * @sa novas_app_to_hor()
- * @sa make_frame()
+ * @sa novas_sky_pos(), novas_app_to_hor()
+ * @sa novas_make_frame()
  */
 int novas_approx_sky_pos(enum novas_planet id, const novas_frame *restrict frame, enum novas_reference_system sys, sky_pos *restrict out) {
   static const char *fn = "novas_approx_sky_pos";
@@ -519,6 +566,11 @@ int novas_approx_sky_pos(enum novas_planet id, const novas_frame *restrict frame
  * a few degrees.</li>
  * </ol>
  *
+ * NOTES:
+ * <ol>
+ * <li>This function caches the result of the last calculation.</li>
+ * </ol>
+ *
  * REFERENCES:
  * <ol>
  *  <li>The Explanatory Supplement to the Astronomical Almanac, University Science Books, 3rd ed.,
@@ -540,13 +592,16 @@ int novas_approx_sky_pos(enum novas_planet id, const novas_frame *restrict frame
  * @since 1.4
  * @author Attila Kovacs
  *
- * @sa novas_next_moon_phase()
- * @sa novas_make_moon_orbit()
- * @sa novas_solar_illum()
+ * @sa novas_next_moon_phase(), novas_make_moon_orbit(), novas_solar_illum()
  */
 double novas_moon_phase(double jd_tdb) {
   static const char *fn = "novas_moon_phase";
 
+  static THREAD_LOCAL double last_tdb = NAN, last_phase;
+
+  if(novas_time_equals(jd_tdb, last_tdb))
+    return last_phase;
+
   novas_orbital orbit = NOVAS_ORBIT_INIT;
   double pos[3] = {0.0};
   double he, hm;
@@ -561,7 +616,10 @@ double novas_moon_phase(double jd_tdb) {
   prop_nan(fn, novas_orbit_native_posvel(jd_tdb, &orbit, pos, NULL));
   vector2radec(pos, &hm, NULL);
 
-  return remainder(12.0 + hm - he, 24.0) * 15.0;
+  last_phase = remainder(12.0 + hm - he, 24.0) * 15.0;
+  last_tdb = jd_tdb;
+
+  return last_phase;
 }
 
 /**
@@ -602,8 +660,7 @@ double novas_moon_phase(double jd_tdb) {
  * @since 1.4
  * @author Attila Kovacs
  *
- * @sa novas_moon_phase()
- * @sa novas_make_moon_orbit()
+ * @sa novas_moon_phase(), novas_make_moon_orbit()
  */
 double novas_next_moon_phase(double phase, double jd_tdb) {
   static const char *fn = "novas_next_moon_phase";
@@ -637,3 +694,10 @@ double novas_next_moon_phase(double phase, double jd_tdb) {
   novas_error(-1, ECANCELED, fn, "Failed to converge");
   return NAN;
 }
+
+
+#if __cplusplus
+#  ifdef NOVAS_NAMESPACE
+} // namespace novas
+#  endif
+#endif
