@@ -1,28 +1,32 @@
 /**
  * @file
  *
- * @date Created  on Jun 27, 2024
- * @author Attila Kovacs and G. Kaplan
- *
  *  A collection of refraction models and utilities to use with novas_app_to_hor() or
  *  novas_hor_to_app().
  *
- * @sa novas_app_to_hor()
- * @sa novas_hor_to_app().
+ * @date Created  on Jun 27, 2024
+ * @author Attila Kovacs and G. Kaplan
+ *
+ * @sa novas_app_to_hor(), novas_hor_to_app()
  */
 
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <errno.h>
+
 /// \cond PRIVATE
 #define __NOVAS_INTERNAL_API__      ///< Use definitions meant for internal use by SuperNOVAS only
 /// \endcond
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <string.h>
-#include <errno.h>
-
 #include "novas.h"
 
+#if __cplusplus
+#  ifdef NOVAS_NAMESPACE
+namespace novas {
+#  endif
+#endif
+
 #define NOVAS_DEFAULT_WAVELENGTH      0.55            ///< [&mu;m] Median wavelength of visible light.
 
 static double lambda = NOVAS_DEFAULT_WAVELENGTH;      ///< [&mu;m] Observing wavelength
@@ -55,8 +59,7 @@ static double lambda = NOVAS_DEFAULT_WAVELENGTH;      ///< [&mu;m] Observing wav
  * @return              [deg] the calculated optical refraction. (to ~0.1 arcsec accuracy),
  *                      or 0.0 if the location is NULL or the option is invalid.
  *
- * @sa refract()
- * @sa itrs_to_hor()
+ * @sa refract(), itrs_to_hor()
  *
  * @since 1.0
  * @author Attila Kovacs
@@ -120,16 +123,11 @@ double refract_astro(const on_surface *restrict location, enum novas_refraction_
  * @return              [deg] the calculated optical refraction or 0.0 if the location is
  *                      NULL or the option is invalid or the 'zd_obs' is invalid (&lt;90&deg;).
  *
- * @sa refract_astro()
- * @sa hor_to_itrs()
- *
+ * @sa refract_astro(), hor_to_itrs()
  */
 double refract(const on_surface *restrict location, enum novas_refraction_model model, double zd_obs) {
   static const char *fn = "refract";
 
-  // 's' is the approximate scale height of atmosphere in meters.
-  const double s = 9.1e3;
-  const double ct = 0.065;  // [C/m] averate temperature drop with altitude
   double p, t, h, r;
 
   if(model < 0 || model >= NOVAS_REFRACTION_MODELS) {
@@ -174,11 +172,10 @@ double refract(const on_surface *restrict location, enum novas_refraction_model
     t = location->temperature;
   }
   else {
-    p = 1010.0 * exp(-location->height / s);
-    // AK: A very rough model of mean annual temperatures vs latitude
-    t = 30.0 - 30.0 * sin(location->latitude * DEGREE);
-    // AK: Estimated temperature drop due to elevation.
-    t -= location->height * ct;
+    on_surface site = *location;
+    novas_set_default_weather(&site);
+    p = site.pressure;
+    t = site.temperature;
   }
 
   h = 90.0 - zd_obs;
@@ -211,15 +208,15 @@ static double novas_refraction(enum novas_refraction_model model, const on_surfa
  *
  * @param model     The original refraction model
  * @param jd_tt     [day] Terrestrial Time (TT) based Julian data of observation
- * @param loc       Pointer to structure defining the observer's location on earth, and local weather
+ * @param loc       Pointer to structure defining the observer's location on earth, and local
+ *                  weather
  * @param type      Refraction type to use for the original model: NOVAS_REFRACT_OBSERVED (-1) or
  *                  NOVAS_REFRACT_ASTROMETRIC (0).
  * @param el0       [deg] input elevation for the inverse refraction model.
  * @return          [deg] Estimated refraction, or NAN if there was an error (it should also
  *                  set errno to indicate the type of error).
  *
- * @sa refract_astro()
- * @sa itrs_to_hor()
+ * @sa refract_astro(), itrs_to_hor()
  *
  * @since 1.1
  * @author Attila Kovacs
@@ -250,19 +247,18 @@ double novas_inv_refract(RefractionModel model, double jd_tt, const on_surface *
 /**
  * Returns an optical refraction correction for a standard atmosphere.
  *
- * @param jd_tt     [day] Terrestrial Time (TT) based Julian data of observation (unused in this implementation of RefractionModel)
- * @param loc       Pointer to structure defining the observer's location on earth, and local weather
- * @param type      Whether the input elevation is observed or astrometric: NOVAS_REFRACT_OBSERVED (-1) or
- *                  NOVAS_REFRACT_ASTROMETRIC (0).
+ * @param jd_tt     [day] Terrestrial Time (TT) based Julian data of observation (unused in this
+ *                  implementation of RefractionModel)
+ * @param loc       Pointer to structure defining the observer's location on earth, and local
+ *                  weather
+ * @param type      Whether the input elevation is observed or astrometric: NOVAS_REFRACT_OBSERVED
+ *                  (-1) or NOVAS_REFRACT_ASTROMETRIC (0).
  * @param el        [deg] Astrometric (unrefracted) source elevation
  * @return          [deg] Estimated refraction, or NAN if there was an error (it should also
  *                  set errno to indicate the type of error).
  *
- * @sa novas_app_to_hor()
- * @sa novas_optical_refraction()
- * @sa NOVAS_STANDARD_ATMOSPHERE()
- * @sa refract()
- * @sa refract_astro()
+ * @sa novas_optical_refraction(), novas_radio_refraction(), novas_wave_refraction()
+ * @sa refract(), refract_astro()
  */
 double novas_standard_refraction(double jd_tt, const on_surface *loc, enum novas_refraction_type type, double el) {
   double dz = novas_refraction(NOVAS_STANDARD_ATMOSPHERE, loc, type, el);
@@ -274,21 +270,23 @@ double novas_standard_refraction(double jd_tt, const on_surface *loc, enum novas
 }
 
 /**
- * Returns an optical refraction correction using the weather parameters defined for the observer location.
+ * Returns an optical refraction correction using the weather parameters defined for the observer
+ * location. As such, make sure that temperature and pressure are defined, e.g. set after
+ * calling e.g. `make_gps_site()`, `make_itrf_site(), `make_xyz_site()`, or similar call that
+ * initializes the observing site.
  *
- * @param jd_tt     [day] Terrestrial Time (TT) based Julian data of observation (unused in this implementation of RefractionModel)
- * @param loc       Pointer to structure defining the observer's location on earth, and local weather
- * @param type      Whether the input elevation is observed or astrometric: NOVAS_REFRACT_OBSERVED (-1) or
- *                  NOVAS_REFRACT_ASTROMETRIC (0).
+ * @param jd_tt     [day] Terrestrial Time (TT) based Julian data of observation (unused in this
+ *                  implementation of RefractionModel)
+ * @param loc       Pointer to structure defining the observer's location on earth, and local
+ *                  weather values (temperature and pressure are used by this call).
+ * @param type      Whether the input elevation is observed or astrometric: NOVAS_REFRACT_OBSERVED
+ *                  (-1) or NOVAS_REFRACT_ASTROMETRIC (0).
  * @param el        [deg] Astrometric (unrefracted) source elevation
  * @return          [arcsec] Estimated refraction, or NAN if there was an error (it should also
  *                  set errno to indicate the type of error).
  *
- * @sa novas_app_to_hor()
- * @sa novas_optical_refraction()
- * @sa NOVAS_STANDARD_ATMOSPHERE()
- * @sa refract()
- * @sa refract_astro()
+ * @sa novas_wave_refraction(), novas_radio_refraction(), novas_standard_refraction()
+ * @sa refract(), refract_astro()
  */
 double novas_optical_refraction(double jd_tt, const on_surface *loc, enum novas_refraction_type type, double el) {
   double dz = novas_refraction(NOVAS_WEATHER_AT_LOCATION, loc, type, el);
@@ -300,10 +298,13 @@ double novas_optical_refraction(double jd_tt, const on_surface *loc, enum novas_
 }
 
 /**
- * Atmospheric refraction model for radio wavelengths (Berman &amp; Rockwell 1976). It uses the
- * weather parameters defined for the location, including humidity. As such, make sure the weather
- * data is fully defined, and that the humidity was explicitly set after calling
- * `make_on_surface()`.
+ * Atmospheric refraction model for radio wavelengths (Berman &amp; Rockwell 1976).
+ *
+ *
+ * It uses the weather parameters defined for the location, including humidity. As such, make sure
+ * the weather data is fully defined, and that the humidity was explicitly set after calling e.g.
+ * `make_gps_site()`, `make_itrf_site(), `make_xyz_site()`, or similar call that initializes the
+ * observing site.
  *
  * Adapted from FORTAN code provided by Berman &amp; Rockwell 1976.
  *
@@ -312,21 +313,21 @@ double novas_optical_refraction(double jd_tt, const on_surface *loc, enum novas_
  * <li>Berman, Allan L., and Rockwell, Stephen T. (1976), NASA JPL Technical Report 32-1601</li>
  * </ol>
  *
- * @param jd_tt     [day] Terrestrial Time (TT) based Julian data of observation (unused in this implementation of RefractionModel)
- * @param loc       Pointer to structure defining the observer's location on earth, and local weather.
- *                  Make sure all weather values, including humidity (added in v1.1), are fully
- *                  populated.
- * @param type      Whether the input elevation is observed or astrometric: NOVAS_REFRACT_OBSERVED (-1) or
- *                  NOVAS_REFRACT_ASTROMETRIC (0).
+ * @param jd_tt     [day] Terrestrial Time (TT) based Julian data of observation (unused in this
+ *                  implementation of RefractionModel)
+ * @param loc       Pointer to structure defining the observer's location on earth, and local
+ *                  weather. Make sure all weather values, including humidity (added in v1.1), are
+ *                  fully populated.
+ * @param type      Whether the input elevation is observed or astrometric: NOVAS_REFRACT_OBSERVED
+ *                  (-1) or NOVAS_REFRACT_ASTROMETRIC (0).
  * @param el        [deg] source elevation of the specified type.
  * @return          [deg] Estimated refraction, or NAN if there was an error (it should also
- *                  set errno to indicate the type of error). An error is returned if the location is
- *                  NULL, or if the weather parameters are way outside of their resonable ranges, or
- *                  if the elevation is outside the supported [-1:90] range.
+ *                  set errno to indicate the type of error). An error is returned if the location
+ *                  is NULL, or if the weather parameters are way outside of their resonable
+ *                  ranges, or if the elevation is outside the supported [-1:90] range.
  *
- * @sa novas_optical_refraction()
- * @sa make_on_surface()
- * @sa on_surface
+ * @sa novas_optical_refraction(), novas_wave_refraction(), novas_standard_refraction()
+ * @sa make_itrf_site(), make_gps_site(), make_xyz_site(), make_itrf_observer(), make_gps_observer()
  */
 double novas_radio_refraction(double jd_tt, const on_surface *loc, enum novas_refraction_type type, double el) {
   static const char *fn = "novas_radio_refraction";
@@ -399,18 +400,17 @@ double novas_radio_refraction(double jd_tt, const on_surface *loc, enum novas_re
 }
 
 /**
- * Sets the observing wavelength for which refraction is to be calculated when using a wavelength-depenendent
- * model, such as novas_wave_refraction().
+ * Sets the observing wavelength for which refraction is to be calculated when using a
+ * wavelength-depenendent model, such as novas_wave_refraction().
  *
  * @param microns     [&mu;m] Observed wavelength to assume in refraction calculations
- * @return            0 if successful, or else -1 (errno set to `EINVAL`) if the wavelength invalid
- *                    (zero, negative, or NaN).
+ * @return            0 if successful, or else -1 (errno set to `EINVAL`) if the wavelength
+ *                    invalid (zero, negative, or NaN).
  *
  * @since 1.4
  * @author Attila Kovacs
  *
- * @sa novas_wave_refraction()
- * @sa NOVAS_DEFAULT_WAVELENGTH
+ * @sa novas_wave_refraction(), NOVAS_DEFAULT_WAVELENGTH
  */
 int novas_refract_wavelength(double microns) {
   if(microns <= 0.0 || microns != microns)
@@ -420,9 +420,10 @@ int novas_refract_wavelength(double microns) {
 }
 
 /**
- * The wavelength-dependent IAU atmospheric refraction model, based on the SOFA `iauRefco()` function, in
- * compliance to the 'SOFA Software License' terms of the original source. Our implementation is not
- * provided nor it is endorsed by SOFA. The original function has been modified slightly, such as:
+ * The wavelength-dependent IAU atmospheric refraction model, based on the SOFA `iauRefco()`
+ * function, in compliance to the 'SOFA Software License' terms of the original source. Our
+ * implementation is not provided nor it is endorsed by SOFA. The original function has been
+ * modified slightly, such as:
  *
  * <ol>
  * <li>Out-of-range weather parameters will return with an error (`errno` set to `EINVAL`), unlike
@@ -432,15 +433,17 @@ int novas_refract_wavelength(double microns) {
  * implementation returns the refraction correction angle.</li>
  * </ol>
  *
- * The refraction is calculated for the observing wavelenth previously set via `novas_refract_wavelength()`,
- * or for visible light at 550 nm by default.
+ * The refraction is calculated for the observing wavelenth previously set via
+ * `novas_refract_wavelength()`, or for visible light at 550 nm by default.
  *
- * The function uses the weather parameters defined for the location, including humidity. As such, make sure
- * the weather data is fully defined, and that the humidity was explicitly set after calling
- * `make_on_surface()`.
+ * The function uses the weather parameters defined for the location, including humidity. As such,
+ * make sure the weather data is fully defined, and that the humidity was explicitly set after
+ * calling e.g. `make_gps_site()`, `make_itrf_site(), `make_xyz_site()`, or similar call that
+ * initializes the observing site.
  *
- * According to the documentation of SOFA's `iauRefco()` function, the model has the following accuracy
- * for elevation angles between 15 and 75 degrees, under a range of typical surface conditions:
+ * According to the documentation of SOFA's `iauRefco()` function, the model has the following
+ * accuracy for elevation angles between 15 and 75 degrees, under a range of typical surface
+ * conditions:
  *
  *   |                  |    worst   |     RMS    |
  *   |------------------|:----------:|:----------:|
@@ -451,12 +454,12 @@ int novas_refract_wavelength(double microns) {
  * NOTES:
  * <ol>
  * <li>
- * From the SOFA documentation: "The model balances speed and accuracy to give good results in applications
- * where performance at low altitudes is not paramount. Performance is maintained across a range of
- * conditions, and applies to both optical/IR and radio."
+ * From the SOFA documentation: "The model balances speed and accuracy to give good results in
+ * applications where performance at low altitudes is not paramount. Performance is maintained
+ * across a range of conditions, and applies to both optical/IR and radio."
  * </li>
- * <li>The model is divergent in the observed direction of the horizon. As such, it should not be used
- * for calculating refraction at or below the horizon itself.</li>
+ * <li>The model is divergent in the observed direction of the horizon. As such, it should not be
+ * used for calculating refraction at or below the horizon itself.</li>
  * </ol>
  *
  * REFERENCES:
@@ -489,12 +492,13 @@ int novas_refract_wavelength(double microns) {
  * </li>
  * </ol>
  *
- * @param jd_tt     [day] Terrestrial Time (TT) based Julian data of observation (unused in this implementation of RefractionModel)
- * @param loc       Pointer to structure defining the observer's location on earth, and local weather.
- *                  Make sure all weather values, including humidity (added in v1.1), are fully
+ * @param jd_tt     [day] Terrestrial Time (TT) based Julian data of observation (unused in this
+ *                  implementation of RefractionModel)
+ * @param loc       Pointer to structure defining the observer's location on earth, and local
+ *                  weather. Make sure all weather values, including humidity (added in v1.1), are fully
  *                  populated.
- * @param type      Whether the input elevation is observed or astrometric: NOVAS_REFRACT_OBSERVED (-1) or
- *                  NOVAS_REFRACT_ASTROMETRIC (0).
+ * @param type      Whether the input elevation is observed or astrometric: NOVAS_REFRACT_OBSERVED
+ *                  (-1) or NOVAS_REFRACT_ASTROMETRIC (0).
  * @param el        [deg] observed source elevation of the specified type.
  * @return          [deg] Estimated refraction, or NAN if there was an error (it should also
  *                  set errno to indicate the type of error), e.g. because the location is NULL, or
@@ -505,11 +509,8 @@ int novas_refract_wavelength(double microns) {
  * @since 1.4
  * @author Attila Kovacs
  *
- * @sa novas_refract_wavelength()
- * @sa novas_optical_refraction()
- * @sa novas_radio_refraction()
- * @sa make_on_surface()
- * @sa on_surface
+ * @sa novas_refract_wavelength(), novas_optical_refraction(), novas_radio_refraction()
+ * @sa make_gps_site(), make_itrf_site(), make_xyz_site(), make_gps_observer(), make_itrf_observer()
  */
 double novas_wave_refraction(double jd_tt, const on_surface *loc, enum novas_refraction_type type, double el) {
   static const char *fn = "novas_wave_refration";
@@ -588,3 +589,9 @@ double novas_wave_refraction(double jd_tt, const on_surface *loc, enum novas_ref
 
   return tanz * (a + b * tanz * tanz) / DEGREE;
 }
+
+#if __cplusplus
+#  ifdef NOVAS_NAMESPACE
+} // namespace novas
+#  endif
+#endif
