@@ -1,10 +1,7 @@
 /**
  * @file
  *
- * @author A. Kovacs
- * @since 1.2
- *
- *  SuperNOVAS Solar-system ephemeris lookup implementation via the NAIF CSPICE library.
+ *  Solar-system ephemeris lookup via the NAIF CSPICE library.
  *
  *  This is an optional component of SuperNOVAS, which interfaces to the NAIF CSPICE Toolkit.
  *  As such, you may need the CSPICE runtime libraries in an accessible location (such
@@ -16,28 +13,112 @@
  *  `Smithsonian/cspice-sharedlib` repository on Github to facilitate the building of CSPICE
  *  as a shared library instead of the static library built by the uptream distribution.
  *
+ *  To use, simply include `novas-cspice.h` in your application source, load the requisite
+ *  SPICE kernels (ephemeris data and more) using `cspice_add_kernel()`, and then activate
+ *  them with `novas_use_cspice()`. E.g.,
+ *
+ *  ```c
+ *    #include <novas.h>
+ *    #include <novas-cspice.h>
+ *
+ *    // You can load the desired kernels for CSPICE
+ *    // E.g. load DE440s and the Mars satellites:
+ *    int status;
+ *
+ *    status = cspice_add_kernel("/path/to/de440s.bsp");
+ *    if(status < 0) {
+ *      // oops, the kernels must not have loaded...
+ *      ...
+ *    }
+ *
+ *    // Load additional kernels as needed...
+ *    status = cspice_add_kernel("/path/to/mar097.bsp");
+ *    ...
+ *
+ *    // Then use CSPICE as your SuperNOVAS ephemeris provider
+ *    novas_use_cspice();
+ *  ```
+ *
+ *  The CSPICE plugin will use the ID numbers stored in the @ref object structure. For planets, it
+ *  will automatically translate NOVAS planet IDs to NAIF IDs, while for other Solar-system
+ *  bodies, you should set the NAIF ID when defining the object via `make_ephem_object()`. If the
+ *  ID is set to -1, then name-based lookup will be used instead.
+ *
  *  REFERENCES:
  *  <ol>
  *  <li>The NAIF CSPICE Toolkit: https://naif.jpl.nasa.gov/naif/toolkit.html</li>
  *  <li>the Smithsonian/cspice-sharedlib repository: https://github.com/Smithsonian/cspice-sharedlib</li>
  *  </ol>
  *
- * @sa solarsystem.h
- * @sa solsys-calceph.c
+ * @author Attila Kovacs
+ * @since 1.2
+ *
+ * @sa solsys-calceph.c, ephemeris.c, orbital.c
  */
 
+#include <stdio.h>
 #include <string.h>
 #include <errno.h>
-#include <semaphore.h>
 
 /// \cond PRIVATE
+#if defined(SUPERNOVAS_USE_THREAD) || defined(__unix__) || defined(__unix) || defined(__APPLE__)
+#  include <pthread.h>
+
+#  define init_lock           pthread_mutex_init
+#  define ephem_lock          pthread_mutex_lock
+#  define ephem_unlock        pthread_mutex_unlock
+#  define THREAD_SAFE         1
+
+typedef pthread_mutex_t       lock_type;
+
+#elif __STDC_VERSION__ >= 201112L
+#  include <threads.h>
+
+#  define init_lock           mtx_init
+#  define ephem_lock          mtx_lock
+#  define ephem_unlock        mtx_unlock
+
+#  define THREAD_SAFE         1
+
+typedef mtx_t                 lock_type;
+
+#elif defined(WIN32)
+#include <windows.h>
+
+#  define init_lock(x, t)     InitializeSRWLock(x)
+#  define ephem_lock          AcquireSRWLockExclusive
+#  define ephem_unlock        ReleaseSRWLockExclusive
+#  define THREAD_SAFE         1
+
+typedef SRWLOCK               lock_type;
+
+#else
+#  define ephem_lock(x)
+#  define ephem_unlock(x)
+#  define THREAD_SAFE         0
+
+typedef int                   lock_type;
+
+#endif
+
+
 #define __NOVAS_INTERNAL_API__      ///< Use definitions meant for internal use by SuperNOVAS only
+/// \endcond
+
 #include "novas.h"
 #include "novas-cspice.h"
 
 #include "cspice/SpiceUsr.h"
 #include "cspice/SpiceZpr.h"        // for reset_c
 
+#if __cplusplus
+#  ifdef NOVAS_NAMESPACE
+namespace novas {
+#  endif
+#endif
+
+/// \cond PRIVATE
+
 /// Multiplicative normalization for the positions returned by km to AU
 #define NORM_POS                    (NOVAS_KM / NOVAS_AU)
 
@@ -45,28 +126,35 @@
 #define NORM_VEL                    (NORM_POS * 86400.0)
 /// \endcond
 
+#if THREAD_SAFE
 /// Semaphore for thread-safe access of ephemerides
-static sem_t *sem;
+static lock_type mutex;
 
-static int mutex_lock() {
-  if(!sem) {
-    sem = (sem_t *) calloc(1, sizeof(sem_t));
-    if(!sem) {
-      perror("ERROR! solsys-cspice: alloc sem_t");
-      exit(errno);
-    }
-    sem_init(sem, 0, 1);
+static void mutex_lock() {
+  static int initialized;
+
+  if(!initialized) {
+    init_lock(&mutex, 0);
+    initialized = 1;
   }
 
-  if(sem_wait(sem) != 0)
-    return novas_error(-1, errno, "mutex_lock()", "sem_wait()");
-
-  return 0;
+  ephem_lock(&mutex);
 }
 
-static int mutex_unlock() {
-  sem_post(sem);
-  return 0;
+static void mutex_unlock() {
+  ephem_unlock(&mutex);
+}
+#endif
+
+/**
+ * Checks if the CSPICE plugin is thread safe.
+ *
+ * @return      TRUE (1) if the lugin is thread safe, or else FALSE (0).
+ *
+ * @since 1.5
+ */
+int novas_cspice_is_thread_safe() {
+  return THREAD_SAFE;
 }
 
 /**
@@ -82,8 +170,8 @@ static int mutex_unlock() {
  * @return 0
  */
 static int suppress_cspice_errors() {
-  erract_c("SET", 0, "RETURN");       // Do not exit in case of CSPICE errors.
-  errprt_c("SET", 0, "NONE");         // Suppress CSPICE error messages
+  erract_c("SET", 0, (SpiceChar *) "RETURN");       // Do not exit in case of CSPICE errors.
+  errprt_c("SET", 0, (SpiceChar *) "NONE");         // Suppress CSPICE error messages
   return 0;
 }
 
@@ -118,11 +206,10 @@ static int get_cspice_error(char *msg, int len) {
  * @return              0 if successful, or else -1 if there was an error (errno will be set to
  *                      EINVAL).
  *
- * @sa cspice_remove_kernel()
- * @sa novas_use_cspice()
- *
  * @author Attila Kovacs
  * @since 1.2
+ *
+ * @sa cspice_remove_kernel(), novas_use_cspice()
  */
 int cspice_add_kernel(const char *filename) {
   static const char *fn = "cspice_add_kernel";
@@ -137,7 +224,7 @@ int cspice_add_kernel(const char *filename) {
 
   suppress_cspice_errors();
 
-  prop_error(fn, mutex_lock(), 0);
+  mutex_lock();
   reset_c();
   furnsh_c(filename);
   err = get_cspice_error(msg, sizeof(msg));
@@ -165,10 +252,10 @@ int cspice_add_kernel(const char *filename) {
  * @return              0 if successful, or else -1 if there was an error (errno will be set to
  *                      EINVAL).
  *
- * @sa cspice_add_kernel()
- *
  * @author Attila Kovacs
  * @since 1.2
+ *
+ * @sa cspice_add_kernel()
  */
 int cspice_remove_kernel(const char *filename) {
   static const char *fn = "cspice_remove_kernel";
@@ -183,7 +270,7 @@ int cspice_remove_kernel(const char *filename) {
 
   suppress_cspice_errors();
 
-  prop_error(fn, mutex_lock(), 0);
+  mutex_lock();
   reset_c();
   unload_c(filename);
   err = get_cspice_error(msg, sizeof(msg));
@@ -201,12 +288,12 @@ int cspice_remove_kernel(const char *filename) {
  * novas_use_cspice() or novas_use_cspice_planet() to activate CSPICE as the NOVAS ephemeris
  * provider.
  *
- * This call is generally thread safe (notwithstanding outside access to the ephemeris files),
+ * This call is generally th#if defined(_PTHREAD_) || defined(__unix__) || defined(__unix) || defined(__APPLE__)read safe (notwithstanding outside access to the ephemeris files),
  * even if CSPICE itself may not be. All ephemeris access will be mutexed to ensure sequential
  * access under the hood.
  *
  * The call will use whatever ephemeris (SPK) files were loaded by the CSPICE library prior
- * to the call (see furnsh_c() function)
+ * to the call (see cspice_add_kernel(), or the CSPICE furnsh_c() functions)
  *
  * REFERENCES:
  * <ol>
@@ -232,12 +319,10 @@ int cspice_remove_kernel(const char *filename) {
  *                       'origin' is invalid, or 3 if there was an error providing ephemeris
  *                       data.
  *
- * @sa planet_cspice
- * @sa novas_use_cspice()
- * @sa novas_use_cspice_planet()
- *
  * @author Attila Kovacs
  * @since 1.2
+ *
+ * @sa planet_cspice(), novas_use_cspice(), novas_use_cspice_planet(), cspice_add_kernel()
  */
 static short planet_cspice_hp(const double jd_tdb[restrict 2], enum novas_planet body, enum novas_origin origin,
         double *restrict position, double *restrict velocity) {
@@ -270,7 +355,7 @@ static short planet_cspice_hp(const double jd_tdb[restrict 2], enum novas_planet
 
   tdb2000 = (jd_tdb[0] + jd_tdb[1] - NOVAS_JD_J2000) * 86400.0;
 
-  prop_error(fn, mutex_lock(), 0);
+  mutex_lock();
 
   // Try with proper planet center NAIF ID first...
   //
@@ -306,14 +391,15 @@ static short planet_cspice_hp(const double jd_tdb[restrict 2], enum novas_planet
 
 /**
  * Provides an interface between the NAIF CSPICE library and NOVAS-C for regular (reduced)
- * precision applications.
+ * precision applications, but in reality it's exactly the same as the high-precision version,
+ * except for the way the TDB-based Julian date is specified.
  *
  * This call is generally thread safe (notwithstanding outside access to the ephemeris files),
  * even if CSPICE itself may not be. All ephemeris access will be mutexed to ensure sequential
  * access under the hood.
  *
  * The call will use whatever ephemeris (SPK) files were loaded by the CSPICE library prior
- * to the call (see furnsh_c() function)
+ * to the call (see cspice_add_kernel(), or the CSPICE furnsh_c() functions)
  *
  * REFERENCES:
  * <ol>
@@ -334,16 +420,12 @@ static short planet_cspice_hp(const double jd_tdb[restrict 2], enum novas_planet
  *                       coordinates in AU referred to the ICRS.
  * @param[out] velocity  [AU/day] Velocity vector of 'body' at jd_tdb; equatorial rectangular
  *                       system referred to the ICRS, in AU/day.
- * @return               0 if successful, or else an error code of solarsystem().
- *
- * @sa planet_eph_manager_hp()
- * @sa planet_ephem_provider()
- * @sa ephem_open()
- * @sa set_planet_provider()
- * @sa solarsystem()
+ * @return               0 if successful, or else an error code defined by nvas_planet_provider.
  *
  * @author Attila Kovacs
  * @since 1.2
+ *
+ * @sa set_planet_provider(), planet_cspice_hp(), cspice_add_kernel()
  */
 static short planet_cspice(double jd_tdb, enum novas_planet body, enum novas_origin origin,
         double *restrict position, double *restrict velocity) {
@@ -376,7 +458,40 @@ static short planet_cspice(double jd_tdb, enum novas_planet body, enum novas_ori
  *                      this may be the integer part of the Julian date for high-precision
  *                      calculations, or else the entire Julian date for reduced precision.
  * @param jd_tdb_low    [day] The low-order part of Barycentric Dynamical Time (TDB) based
- *                      Julian date for which to find the position and velocity. Typically
+ *                      Julian date for which to find #if defined(_PTHREAD_) || defined(__unix__) || defined(__unix) || defined(__APPLE__)
+#  include <pthread.h>
+
+#  define mutex_init      pthread_mutex_init
+#  define mutex_lock      pthread_mutex_lock
+#  define mutex_unlock    pthread_mutex_unlock
+#  define THREAD_SAFE     1
+
+typedef pthread_mutex_t   mtx_t;
+
+#elif __STDC_VERSION__ >= 201112L
+#  include <threads.h>
+#  define mutex_init      mtx_init
+#  define mutex_lock      mtx_lock
+#  define mutex_unlock    mtx_unlock
+
+#  define THREAD_SAFE     1
+
+#elif defined(WIN32)
+#include <windows.h>
+#  define mutex_init      InitializeSRWLock
+#  define mutex_lock      AcquireSRWLockExclusive
+#  define mutex_unlock    ReleaseSRWLockExclusive
+#  define THREAD_SAFE     1
+
+#else
+#  define mtx_lock        (void)
+#  define mtx_unlock      (void)
+#  define THREAD_SAFE     0
+
+typedef int               mtx_t;
+
+#endif
+ *                      the position and velocity. Typically
  *                      this may be the fractional part of the Julian date for high-precision
  *                      calculations, or else 0.0 if the date is defined entirely by the
  *                      high-order component for reduced precision.
@@ -390,13 +505,11 @@ static short planet_cspice(double jd_tdb, enum novas_planet body, enum novas_ori
  *                      non-zero value if the was an error s.t. the position and velocity
  *                      vector should not be used.
  *
- * @sa novas_cspice_use_ids()
- * @sa set_ephem_provider()
- * @sa ephemeris()
- * @sa NOVAS_EPHEM_OBJECT
- *
- *@author Attila Kovacs
+ * @author Attila Kovacs
  * @since 1.2
+ *
+ * @sa novas_cspice_use_ids()
+ * @sa set_ephem_provider(), ephemeris(), make_ephem_object()
  */
 static int novas_cspice(const char *name, long id, double jd_tdb_high, double jd_tdb_low, enum novas_origin *origin, double *pos, double *vel) {
   static const char *fn = "novas_cspice";
@@ -423,12 +536,12 @@ static int novas_cspice(const char *name, long id, double jd_tdb_high, double jd
     bodn2c_c(name, &spiceCode, &spiceFound);
     err = get_cspice_error(msg, sizeof(msg));
 
+    if(err)
+      return novas_error(1, EINVAL, fn, "CSPICE error for '%s': %s", name, msg);
+
     if(!spiceFound)
       return novas_error(1, EINVAL, fn, "CSPICE could not find a NAIF ID for '%s'", name);
 
-    if(err)
-      return novas_error(1, EINVAL, fn, "CSPICE name lookup error for '%s': %s", name, msg);
-
     id = spiceCode;
   }
 
@@ -442,7 +555,7 @@ static int novas_cspice(const char *name, long id, double jd_tdb_high, double jd
 
   tdb2000 = (jd_tdb_high + jd_tdb_low - NOVAS_JD_J2000) * 86400.0;
 
-  prop_error(fn, mutex_lock(), 0);
+  mutex_lock();
 
   // See https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/req/frames.html#Reference%20Frames
   // "J2000" and "ICRF" are treated the same, with "J2000" being the compatibility label.
@@ -475,13 +588,11 @@ static int novas_cspice(const char *name, long id, double jd_tdb_high, double jd
  *
  * @return  0
  *
- * @sa novas_use_cspice_planets()
- * @sa novas_use_cspice()
- * @sa cspice_add_kernel()
- * @sa set_ephem_provider()
- *
  * @author Attila Kovacs
  * @since 1.2
+ *
+ * @sa novas_use_cspice_planets(), novas_use_cspice(), cspice_add_kernel()
+ * @sa set_ephem_provider()
  */
 int novas_use_cspice_ephem() {
   suppress_cspice_errors();
@@ -499,14 +610,13 @@ int novas_use_cspice_ephem() {
  *
  * @return  0
  *
- * @sa novas_use_cspice_ephem()
- * @sa novas_use_cspice()
- * @sa cspice_add_kernel()
- * @sa set_planet_provider()
- * @sa set_planet_provider_hp()
- *
  * @author Attila Kovacs
  * @since 1.2
+ *
+ * @sa novas_use_cspice_ephem(), novas_use_cspice(), cspice_add_kernel()
+ * @sa set_planet_provider(), set_planet_provider_hp()
+ *
+ *
  */
 int novas_use_cspice_planets() {
   suppress_cspice_errors();
@@ -516,8 +626,8 @@ int novas_use_cspice_planets() {
 }
 
 /**
- * Sets CSPICE as the default ephemeris provider for all types of Solar-system objects (both NOVAS_PLANET and
- * NOVAS_EPHEM_OBJECT types).
+ * Sets CSPICE as the default ephemeris provider for all types of Solar-system objects (both
+ * NOVAS_PLANET and NOVAS_EPHEM_OBJECT types).
  *
  * CSPICE is configured to suppress error messages and to not exit on errors, since we will check
  * errors and handle them ourselves. You can adjust the behavior after this call with the
@@ -525,12 +635,10 @@ int novas_use_cspice_planets() {
  *
  * @return  0
  *
- * @sa novas_use_cspice_planets()
- * @sa novas_use_cspice_ephem()
- * @sa cspice_add_kernel()
- *
  * @author Attila Kovacs
  * @since 1.2
+ *
+ * @sa novas_use_cspice_planets(), novas_use_cspice_ephem(), cspice_add_kernel()
  */
 int novas_use_cspice() {
   novas_use_cspice_planets();
@@ -538,5 +646,10 @@ int novas_use_cspice() {
   return 0;
 }
 
+#if __cplusplus
+#  ifdef NOVAS_NAMESPACE
+} // namespace novas
+#  endif
+#endif
 
 
