@@ -1,37 +1,49 @@
 /**
  * @file
  *
- * @date Created  on Mar 6, 2025
- * @author G. Kaplan and A. Kovacs
- *
  *  Various commonly used routines used throughout the SuperNOVAS library.
+ *
+ * @date Created  on Mar 6, 2025
+ * @author G. Kaplan and Attila Kovacs
  */
 
-#include <stdarg.h>
+#include <stdarg.h>               // before stdio for vfprintf on LynxOS 3.1
+#include <stdio.h>
 #include <string.h>
 #include <errno.h>
-#include <unistd.h>
 
 /// \cond PRIVATE
 #define __NOVAS_INTERNAL_API__    ///< Use definitions meant for internal use by SuperNOVAS only
+/// \endcond
+
 #include "novas.h"
+
+#if __cplusplus
+#  ifdef NOVAS_NAMESPACE
+namespace novas {
+#  endif
+#endif
+
+
+/// \cond PRIVATE
+#define MAX_SECONDS_DECIMALS      9                 ///< Maximum decimal places for seconds in HMS/DMS formats
 /// \endcond
 
 /// Current debugging state for reporting errors and traces to stderr.
 static enum novas_debug_mode novas_debug_state = NOVAS_DEBUG_OFF;
 
 /**
- * Maximum number of iterations for convergent inverse calculations. Most iterative inverse functions should
- * normally converge in a handful of iterations. In some pathological cases more iterations may be required.
- * This variable sets an absolute maximum for the number of iterations in order to avoid runaway (zombie)
- * behaviour. If inverse functions faile to converge, they will return a value indicating an error, and
- * errno should be set to ECANCELED.
+ * Maximum number of iterations for convergent inverse calculations. Most iterative inverse
+ * functions should normally converge in a handful of iterations. In some pathological cases more
+ * iterations may be required. This variable sets an absolute maximum for the number of iterations
+ * in order to avoid runaway (zombie) behaviour. If inverse functions fail to converge, they will
+ * return a value indicating an error, and errno should be set to ECANCELED.
  *
  * @since 1.1
  */
 int novas_inv_max_iter = 100;
 
-/// \cond PRIVATE
+/// \cond PROTECTED
 
 /**
  * (<i>for internal use</i>) Propagates an error (if any) with an offset. If the error is
@@ -42,6 +54,9 @@ int novas_inv_max_iter = 100;
  * @param n       error code that was received.
  * @param offset  Offset to add to error code (if &lt;0) to return.
  *
+ * @since 1.0
+ * @author Attila Kovacs
+ *
  * @sa novas_error()
  */
 int novas_trace(const char *restrict loc, int n, int offset) {
@@ -60,6 +75,7 @@ int novas_trace(const char *restrict loc, int n, int offset) {
  * @return       NAN
  *
  * @since 1.1.1
+ * @author Attila Kovacs
  */
 double novas_trace_nan(const char *restrict loc) {
   if(novas_get_debug_mode() != NOVAS_DEBUG_OFF) {
@@ -76,11 +92,10 @@ double novas_trace_nan(const char *restrict loc) {
  * @param from  {string} Function (:location) where error originated
  * @param desc  {string} Description of error, with information to convey to user.
  *
- * @sa novas_error()
- * @sa novas_debug()
- *
  * @since 1.0
- * @author Attil Kovacs
+ * @author Attila Kovacs
+ *
+ * @sa novas_error(), novas_debug()
  */
 void novas_set_errno(int en, const char *restrict from, const char *restrict desc, ...) {
   va_list varg;
@@ -105,11 +120,10 @@ void novas_set_errno(int en, const char *restrict from, const char *restrict des
  * @param from  function (:location) where error originated
  * @param desc  description of error, with information to convey to user.
  *
- * @sa novas_set_errno()
- * @sa novas_trace()
- *
  * @since 1.0
  * @author Attila Kovacs
+ *
+ * @sa novas_set_errno(), novas_trace()
  */
 int novas_error(int ret, int en, const char *restrict from, const char *restrict desc, ...) {
   va_list varg;
@@ -134,11 +148,10 @@ int novas_error(int ret, int en, const char *restrict from, const char *restrict
  *              be NULL
  * @return      The distance squared between the two vectors
  *
- * @sa vlen()
- * @sa vdot()
- *
  * @since 1.3
  * @author Attila Kovacs
+ *
+ * @sa novas_vlen(), novas_vdot()
  */
 double novas_vdist2(const double *v1, const double *v2) {
   double d2 = 0.0;
@@ -158,11 +171,10 @@ double novas_vdist2(const double *v1, const double *v2) {
  *              be NULL
  * @return      The distance between the two vectors
  *
- * @sa vlen()
- * @sa vdot()
- *
  * @since 1.0
  * @author Attila Kovacs
+ *
+ * @sa novas_vlen(), novas_vdot()
  */
 double novas_vdist(const double *v1, const double *v2) {
   return sqrt(novas_vdist2(v1, v2));
@@ -176,11 +188,10 @@ double novas_vdist(const double *v1, const double *v2) {
  *              be NULL
  * @return      The dot product between the two vectors.
  *
- * @sa vlen()
- * @sa vdist()
- *
  * @since 1.0
  * @author Attila Kovacs
+ *
+ * @sa novas_vlen(), novas_vdist()
  */
 double novas_vdot(const double *v1, const double *v2) {
   return (v1[0] * v2[0]) + (v1[1] * v2[1]) + (v1[2] * v2[2]);
@@ -196,20 +207,22 @@ double novas_vdot(const double *v1, const double *v2) {
  * for the highest accuracy calculations. As such it is safe to use the reduced accuracy time
  * check for cached precession-related quantities.
  *
- * @param accuracy  NOVAS_FULL_ACCURACY (0) or NOVAS_REDUCED_ACCURACY (1)
  * @param jd1       [day] a Julian date (in any time measure)
  * @param jd2       [day] a Julian date in the same time measure as the first argument
  * @return          TRUE (1) if the two dates are effectively the same at the precision of
  *                  comparison, or else FALSE (0) if they differ by more than the allowed
  *                  tolerance.
+ *
+ * @since 1.0
+ * @author Attila Kovacs
  */
 int novas_time_equals(double jd1, double jd2) {
   return fabs(jd1 - jd2) < 1e-7;
 }
 
 /**
- * (<i>for internal use only</i>) Checks if two Julian dates are precisely equal with regard to the
- * dynamic equator. The two dates are considered equal if they agree within 10<sup>-7</sup>
+ * (<i>for internal use only</i>) Checks if two Julian dates are precisely equal with regard to
+ * the dynamic equator. The two dates are considered equal if they agree within 10<sup>-7</sup>
  * days (or about 10 ms) of each other in reduced accuracy or if they agree within 10<sup>-9</sup>
  * days (or about 100 us), corresponding to double-precision limits, in full accuracy mode.
  *
@@ -224,6 +237,9 @@ int novas_time_equals(double jd1, double jd2) {
  * @return          TRUE (1) if the two dates are effectively the same at the precision of
  *                  comparison, or else FALSE (0) if they differ by more than the allowed
  *                  tolerance.
+ *
+ * @since 1.0
+ * @author Attila Kovacs
  */
 int novas_time_equals_hp(double jd1, double jd2) {
   return fabs(jd1 - jd2) < 1e-9;
@@ -238,6 +254,8 @@ int novas_time_equals_hp(double jd1, double jd2) {
  * @param az            (rad) rotation angle around z
  * @param[out] out      Rotated vector. It can be the same as the input.
  *
+ * @since 1.0
+ * @author Attila Kovacs
  */
 void novas_tiny_rotate(const double *in, double ax, double ay, double az, double *out) {
   const double x = in[0], y = in[1], z = in[2];
@@ -248,14 +266,27 @@ void novas_tiny_rotate(const double *in, double ax, double ay, double az, double
   out[2] = z - 0.5 * (A[0] + A[1]) * z - ay * x + ax * y;
 }
 
-/// \endcond
+/**
+ * Sets the maximum number of iterations allowed for convergent inverese calculations.
+ *
+ * @param n   Maximum number of iterations allowed.
+ *
+ * @since 1.5
+ * @author Attila Kovacs
+ */
+void novas_set_max_iter(int n) {
+  novas_inv_max_iter = n;
+}
 
-/// ==========================================================================
+/// \endcond PROTECTED
+
+// ===========================================================================
 
 /**
  * Enables or disables reporting errors and traces to the standard error stream.
  *
- * @param mode    NOVAS_DEBUG_OFF (0; or &lt;0), NOVAS_DEBUG_ON (1), or NOVAS_DEBUG_EXTRA (2; or &gt;2).
+ * @param mode    NOVAS_DEBUG_OFF (0; or &lt;0), NOVAS_DEBUG_ON (1), or NOVAS_DEBUG_EXTRA (2; or
+ *                &gt;2).
  *
  * @since 1.0
  * @author Attila Kovacs
@@ -308,11 +339,10 @@ double novas_norm_ang(double angle) {
  * @param v     Pointer to a 3-component (x, y, z) vector. The argument cannot be NULL
  * @return      the length of the vector
  *
- * @sa vdot()
- * @sa vdist()
- *
  * @since 1.0
  * @author Attila Kovacs
+ *
+ * @sa novas_vdot(), novas_vdist()
  */
 double novas_vlen(const double *restrict v) {
   return sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2]);
@@ -387,9 +417,8 @@ int spin(double angle, const double *in, double *out) {
  * @param[out] dec  [deg] Declination in degrees [-90:90] or NAN if the position vector is NULL or
  *                  a null-vector. It may be NULL if not required.
  * @return          0 if successful, -1 of any of the arguments are NULL, or
- *                  1 if all input components are 0 so 'ra' and 'dec' are indeterminate,
- *                  or else 2 if both x and y are zero, but z is nonzero, and so 'ra' is
- *                  indeterminate.
+ *                  1 if all input components are 0 so 'ra' and 'dec' are indeterminate, or else 2
+ *                  if both x and y are zero, but z is nonzero, and so 'ra' is indeterminate.
  *
  * @sa radec2vector()
  */
@@ -440,8 +469,7 @@ short vector2radec(const double *restrict pos, double *restrict ra, double *rest
  *
  * @return            0 if successful, or -1 if the vector argument is NULL.
  *
- * @sa vector2radec()
- * @sa starvectors()
+ * @sa vector2radec(), starvectors()
  */
 int radec2vector(double ra, double dec, double dist, double *restrict pos) {
   double cosdec;
@@ -476,7 +504,7 @@ int radec2vector(double ra, double dec, double dist, double *restrict pos) {
  *
  * NOTES:
  * <ol>
- * <li>This function is called by place()</li>
+ * <li>This function is called by `novas_geom_posvel()`, novas_sky_pos(), or `place()`</li>
  * </ol>
  *
  * @param pos_src   Position vector towards observed object, with respect to the SSB
@@ -488,7 +516,7 @@ int radec2vector(double ra, double dec, double dist, double *restrict pos) {
  *                  (<i>Usage A</i>) or relative intermediate solar-system body
  *                  (<i>Usage B</i>); or else NAN if either of the input arguments is NULL.
  *
- * @sa place()
+ * @sa novas_sky_pos(), novas_geom_posvel()
  */
 double d_light(const double *pos_src, const double *pos_body) {
   double d_src;
@@ -517,9 +545,7 @@ double d_light(const double *pos_src, const double *pos_body) {
  * @since 1.3
  * @author Attila Kovacs
  *
- * @sa novas_equ_sep()
- * @sa novas_sun_angle()
- * @sa novas_moon_angle()
+ * @sa novas_equ_sep(), novas_sun_angle(), novas_moon_angle()
  */
 double novas_sep(double lon1, double lat1, double lon2, double lat2) {
   double c = sin(lat1 * DEGREE) * sin(lat2 * DEGREE) + cos(lat1 * DEGREE) * cos(lat2 * DEGREE) * cos((lon1 - lon2) * DEGREE);
@@ -540,10 +566,234 @@ double novas_sep(double lon1, double lat1, double lon2, double lat2) {
  * @since 1.3
  * @author Attila Kovacs
  *
- * @sa novas_sep()
- * @sa novas_sun_angle()
- * @sa novas_moon_angle()
+ * @sa novas_sep(), novas_sun_angle(), novas_moon_angle()
  */
 double novas_equ_sep(double ra1, double dec1, double ra2, double dec2) {
   return novas_sep(15.0 * ra1, dec1, 15.0 * ra2, dec2);
 }
+
+
+/**
+ * Breaks down a value into hours/degrees, minutes, seconds, and a subsecond part given the
+ * number of decimals requested. The last sigit is rounded as appropriate.
+ *
+ * @param value     The input hours or degrees
+ * @param decimals  Number of requested decimals for the sub-second component
+ * @param[out] h    The hours or degrees part
+ * @param[out] m    the minutes components
+ * @param[out] s    the seconds component
+ * @param[out] ss   the subseconds component for the given number of decimals.
+ */
+static void breakdown(double value, int decimals, int *h, int *m, int *s, long long *ss) {
+  long long mult = (long long) pow(10.0, decimals > 0 ? decimals : 0);
+  long long u;
+
+  *ss = (long long) floor(value * 3600L * mult + 0.5);
+
+  u = 3600L * mult;
+  *h = (int) (*ss / u);
+  *ss -= (*h) * u;
+
+  u = 60L * mult;
+  *m = (int) (*ss / u);
+  *ss -= (*m) * u;
+
+  *s = (int) (*ss / mult);
+  *ss -= (*s) * mult;
+}
+
+/**
+ * Prints a time in hours as hh:mm:ss[.S...] into the specified buffer, with up to nanosecond
+ * precision.
+ *
+ * NaN and infinite values, are printed as their standard floating-point representations.
+ *
+ * @param hours       [h] time value
+ * @param sep         Type of separators to use between or after components. If the separator
+ *                    value is outside of the enum range, it will default to using colons.
+ * @param decimals    Requested number of decimal places to print for the seconds [0:9].
+ * @param[out] buf    String buffer in which to print HMS string, represented in the [0:24)
+ *                    hour range.
+ * @param len         Maximum number of bytes that may be written into the output buffer,
+ *                    including termination.
+ * @return            The number of characters actually printed in the buffer, excluding
+ *                    termination, or else -1 if the input buffer is NULL or the length
+ *                    is less than 1 (errno will be set to EINVAL).
+ *
+ * @since 1.3
+ * @author Attila Kovacs
+ *
+ * @sa novas_parse_hms(), novas_print_dms(), novas_timestamp()
+ */
+int novas_print_hms(double hours, enum novas_separator_type sep, int decimals, char *restrict buf, int len) {
+  static const char *fn = "novas_print_hms";
+
+  char tmp[100] = {'\0'};
+
+  if(!buf)
+    return novas_error(-1, EINVAL, fn, "output buffer is NULL");
+
+  if(len < 1)
+    return novas_error(-1, EINVAL, fn, "invalid output buffer len: %d", len);
+
+  if(hours != hours)
+    sprintf(tmp, "%f", hours);      // nan, inf
+  else {
+    int h, m, s;
+    long long ss;
+    char fmt[40];
+    const char *seph, *sepm, *seps;
+
+    if(decimals > MAX_SECONDS_DECIMALS)
+      decimals = MAX_SECONDS_DECIMALS;
+
+    if(decimals > 0)
+      sprintf(fmt, "%%02d%%s%%02d%%s%%02d.%%0%dlld%%s", decimals);
+    else
+      sprintf(fmt, "%%02d%%s%%02d%%s%%02d%%s");
+
+    switch(sep) {
+      case NOVAS_SEP_UNITS:
+        seph = "h";
+        sepm = "m";
+        seps = "s";
+        break;
+
+      case NOVAS_SEP_UNITS_AND_SPACES:
+        seph = "h ";
+        sepm = "m ";
+        seps = "s";
+        break;
+
+      case NOVAS_SEP_SPACES:
+        seph = " ";
+        sepm = " ";
+        seps = "";
+        break;
+
+      case NOVAS_SEP_COLONS:
+      default:
+        seph = ":";
+        sepm = ":";
+        seps = "";
+    }
+
+    // in [0:24h] range
+    hours -= 24.0 * floor(hours / 24.0);
+    breakdown(hours, decimals, &h, &m, &s, &ss);
+
+    if(decimals > 0)
+      sprintf(tmp, fmt, h, seph, m, sepm, s, ss, seps);
+    else
+      sprintf(tmp, fmt, h, seph, m, sepm, s, seps);
+  }
+
+  strncpy(buf, tmp, len - 1);
+  buf[len-1] = '\0';
+
+  return strlen(buf);
+}
+
+/**
+ * Prints an angle in degrees as [-]ddd:mm:ss[.S...] into the specified buffer, with up to
+ * nanosecond precision.
+ *
+ * The degrees component is always printed as 4 characters (up to 3 digits
+ * plus optional negative sign), so the output is always aligned. If positive values are
+ * expected to be explicitly signed also, the caller may simply put the '+' sign into the
+ * leading byte.
+ *
+ * NaN and infinite values, are printed as their standard floating-point representations.
+ *
+ * @param degrees     [deg] angle value
+ * @param sep         Type of separators to use between or after components. If the separator
+ *                    value is outside of the enum range, it will default to using colons.
+ * @param decimals    Requested number of decimal places to print for the seconds [0:9].
+ * @param[out] buf    String buffer in which to print DMS string, represented in the [-180:180)
+ *                    degree range.
+ * @param len         Maximum number of bytes that may be written into the output buffer,
+ *                    including termination.
+ * @return            The number of characters actually printed in the buffer, excluding
+ *                    termination, or else -1 if the input buffer is NULL or the length
+ *                    is less than 1 (errno will be set to EINVAL).
+ *
+ * @since 1.3
+ * @author Attila Kovacs
+ *
+ * @sa novas_parse_dms(), novas_print_hms()
+ */
+int novas_print_dms(double degrees, enum novas_separator_type sep, int decimals, char *restrict buf, int len) {
+  static const char *fn = "novas_print_dms";
+
+  char tmp[40] = {'\0'};
+
+  if(!buf)
+    return novas_error(-1, EINVAL, fn, "output buffer is NULL");
+
+  if(len < 1)
+    return novas_error(-1, EINVAL, fn, "invalid output buffer len: %d", len);
+
+  if(degrees != degrees)
+    sprintf(tmp, "%f", degrees);      // nan, inf
+  else {
+    int d, m, s;
+    long long ss;
+    char fmt[40];
+    const char *sepd, *sepm, *seps;
+
+    if(decimals > MAX_SECONDS_DECIMALS)
+      decimals = MAX_SECONDS_DECIMALS;
+
+    if(decimals > 0)
+      sprintf(fmt, "%%4d%%s%%02d%%s%%02d.%%0%dlld%%s", decimals);
+    else
+      sprintf(fmt, "%%4d%%s%%02d%%s%%02d%%s");
+
+    degrees = remainder(degrees, DEG360);
+    breakdown(degrees, decimals, &d, &m, &s, &ss);
+
+    switch(sep) {
+      case NOVAS_SEP_UNITS:
+        sepd = "d";
+        sepm = "m";
+        seps = "s";
+        break;
+
+      case NOVAS_SEP_UNITS_AND_SPACES:
+        sepd = "d ";
+        sepm = "m ";
+        seps = "s";
+        break;
+
+      case NOVAS_SEP_SPACES:
+        sepd = " ";
+        sepm = " ";
+        seps = "";
+        break;
+
+      case NOVAS_SEP_COLONS:
+      default:
+        sepd = ":";
+        sepm = ":";
+        seps = "";
+    }
+
+    if(decimals > 0)
+      sprintf(tmp, fmt, d, sepd, m, sepm, s, ss, seps);
+    else
+      sprintf(tmp, fmt, d, sepd, m, sepm, s, seps);
+  }
+
+  strncpy(buf, tmp, len - 1);
+  buf[len-1] = '\0';
+
+  return strlen(buf);
+}
+
+
+#if __cplusplus
+#  ifdef NOVAS_NAMESPACE
+} // namespace novas
+#  endif
+#endif
+
