@@ -12,6 +12,15 @@
 
 #define __NOVAS_INTERNAL_API__      ///< Use definitions meant for internal use by SuperNOVAS only
 #include "novas.h"
+#include "solarsystem.h"
+
+#if defined _WIN32 || defined __CYGWIN__
+#  define PATH_SEP  "\\"
+#else
+#  define PATH_SEP  "/"
+#endif
+
+static char *dataPath = "../data";
 
 static int dummy_ephem(const char *name, long id, double jd_tdb_high, double jd_tdb_low, enum novas_origin *origin, double *pos, double *vel) {
   (void) name;
@@ -236,7 +245,6 @@ static int test_refract() {
 }
 
 static int test_refract_astro() {
-  extern int novas_inv_max_iter;
   on_surface surf = ON_SURFACE_INIT;
   int n = 0;
 
@@ -244,26 +252,25 @@ static int test_refract_astro() {
   if(check_nan("refract_astro:model:-1", refract_astro(&surf, -1, 30.0))) n++;
   if(check_nan("refract_astro:model:hi", refract_astro(&surf, NOVAS_REFRACTION_MODELS, 30.0))) n++;
 
-  novas_inv_max_iter = 0;
+  novas_set_max_iter(0);
   if(check_nan("refract_astro:converge", refract_astro(&surf, NOVAS_STANDARD_ATMOSPHERE, 85.0))) n++;
   else if(check("refract_astro:converge:errno", ECANCELED, errno)) n++;
 
-  novas_inv_max_iter = 100;
+  novas_set_max_iter(NOVAS_DEFAULT_MAX_ITER);
 
   return n;
 }
 
 static int test_inv_refract() {
-  extern int novas_inv_max_iter;
   on_surface surf = ON_SURFACE_INIT;
   int n = 0;
 
   if(check_nan("inv_refract:loc", novas_inv_refract(novas_optical_refraction, NOVAS_JD_J2000, NULL, NOVAS_REFRACT_OBSERVED, 5.0))) n++;;
 
-  novas_inv_max_iter = 0;
+  novas_set_max_iter(0);
   if(check_nan("inv_refract:converge", novas_inv_refract(novas_optical_refraction, NOVAS_JD_J2000, &surf, NOVAS_REFRACT_OBSERVED, 5.0))) n++;
   else if(check("inv_refract:converge:errno", ECANCELED, errno)) n++;
-  novas_inv_max_iter = 100;
+  novas_set_max_iter(NOVAS_DEFAULT_MAX_ITER);
 
   return n;
 }
@@ -505,6 +512,28 @@ static int test_cirs_to_gcrs() {
   return n;
 }
 
+static int test_cirs_to_tod() {
+  double p[3];
+  int n = 0;
+
+  if(check("cirs_to_tod:in", -1, cirs_to_tod(0.0, NOVAS_FULL_ACCURACY, NULL, p))) n++;
+  if(check("cirs_to_tod:out", -1, cirs_to_tod(0.0, NOVAS_FULL_ACCURACY, p, NULL))) n++;
+  if(check("cirs_to_tod:accuracy", -1, cirs_to_tod(0.0, -1, p, p))) n++;
+
+  return n;
+}
+
+static int test_tod_to_cirs() {
+  double p[3];
+  int n = 0;
+
+  if(check("tod_to_cirs:in", -1, tod_to_cirs(0.0, NOVAS_FULL_ACCURACY, NULL, p))) n++;
+  if(check("tod_to_cirs:out", -1, tod_to_cirs(0.0, NOVAS_FULL_ACCURACY, p, NULL))) n++;
+  if(check("tod_to_cirs:accuracy", -1, tod_to_cirs(0.0, -1, p, p))) n++;
+
+  return n;
+}
+
 static int test_cirs_to_app_ra() {
   int n = 0;
   if(check_nan("cirs_to_app_ra:accuracy:-1", cirs_to_app_ra(NOVAS_JD_J2000, -1, 0.0))) n++;
@@ -569,17 +598,16 @@ static int test_radec_planet() {
 }
 
 static int test_mean_star() {
-  extern int novas_inv_max_iter;
   double x, y;
   int n = 0;
 
   if(check("mean_star:ira", -1, mean_star(0.0, 0.0, 0.0, NOVAS_FULL_ACCURACY, NULL, &y))) n++;
   if(check("mean_star:idec", -1, mean_star(0.0, 0.0, 0.0, NOVAS_FULL_ACCURACY, &x, NULL))) n++;
 
-  novas_inv_max_iter = 0;
+  novas_set_max_iter(0);
   if(check("mean_star:converge", 1, mean_star(NOVAS_JD_J2000, 0.0, 0.0, NOVAS_REDUCED_ACCURACY, &x, &y))) n++;
   else if(check("mean_star:converge:errno", ECANCELED, errno)) n++;
-  novas_inv_max_iter = 100;
+  novas_set_max_iter(NOVAS_DEFAULT_MAX_ITER);
 
   return n;
 }
@@ -701,7 +729,6 @@ static int test_sidereal_time() {
 
   if(check("sidereal_time:out", -1, sidereal_time(0.0, 0.0, 0.0, NOVAS_MEAN_EQUINOX, EROT_GST, NOVAS_FULL_ACCURACY, NULL))) n++;
   if(check("sidereal_time:accuracy", 1, sidereal_time(0.0, 0.0, 0.0, NOVAS_MEAN_EQUINOX, EROT_GST, -1, &x))) n++;
-  if(check("sidereal_time:erot", 2, sidereal_time(0.0, 0.0, 0.0, NOVAS_MEAN_EQUINOX, -1, NOVAS_FULL_ACCURACY, &x))) n++;
 
   return n;
 }
@@ -832,7 +859,6 @@ static int test_geo_posvel() {
 }
 
 static int test_light_time2() {
-  extern int novas_inv_max_iter;
   object o;
   double pos[3] = {1.0}, p[3], v[3], t;
   int n = 0;
@@ -843,10 +869,10 @@ static int test_light_time2() {
   if(check("light_time2:object", -1, light_time2(0.0, NULL, pos, 0.0, NOVAS_FULL_ACCURACY, p, v, &t))) n++;
   if(check("light_time2:pos", -1, light_time2(0.0, &o, NULL, 0.0, NOVAS_FULL_ACCURACY, p, v, &t))) n++;
 
-  novas_inv_max_iter = 0;
+  novas_set_max_iter(0);
   if(check("light_time2:converge", 1, light_time2(0.0, &o, pos, 0.0, NOVAS_FULL_ACCURACY, p, v, &t))) n++;
   else if(check("light_time2:converge:errno", ECANCELED, errno)) n++;
-  novas_inv_max_iter = 100;
+  novas_set_max_iter(NOVAS_DEFAULT_MAX_ITER);
 
   return n;
 }
@@ -867,46 +893,6 @@ static int test_cio_array() {
 
   if(check("cio_array:out", -1, cio_array(0.0, 5, NULL))) n++;
   if(check("cio_array:n_pts:lo", 3, cio_array(0.0, 1, x))) n++;
-  if(check("cio_array:n_pts:hi", 3, cio_array(0.0, NOVAS_CIO_CACHE_SIZE + 1, x))) n++;
-
-  set_cio_locator_file("blah");
-  if(check("cio_array:file", 1, cio_array(0.0, 5, x))) n++;
-
-  set_cio_locator_file("../cio_ra.bin");
-  set_cio_locator_file("../cio_ra.bin"); // Test reopen also...
-  if(check("cio_array:beg", 2, cio_array(0.0, 5, x))) n++;
-  if(check("cio_array:end", 2, cio_array(1e20, 5, x))) n++;
-
-  if(check("cio_array:corner:lo", 6, cio_array(2341952.6, 5, x))) n++;
-  if(check("cio_array:corner:hi", 6, cio_array(2561137.4, 5, x))) n++;
-
-  if(check("cio_array:corner:near", 0, cio_array(2341962.6, 5, x))) n++;
-
-  set_cio_locator_file("bad-cio-data/empty");
-  if(check("cio_array:bin:empty", 1, cio_array(2341952.6, 5, x))) n++;
-
-  set_cio_locator_file("bad-cio-data/bad-1.bin");
-  if(check("cio_array:bin:header", -1, cio_array(2341952.6, 2, x))) n++;
-
-  set_cio_locator_file("bad-cio-data/bad-2.bin");
-  if(check("cio_array:bin:incomplete", 6, cio_array(2341951.4, 2, x))) n++;
-  if(check("cio_array:bin:seek", -1, cio_array(2341965.4, 2, x))) n++;
-
-  set_cio_locator_file("bad-cio-data/bad-1.txt");
-  if(check("cio_array:ascii:header", -1, cio_array(2341952.6, 2, x))) n++;
-
-  set_cio_locator_file("bad-cio-data/bad-2.txt");
-  if(check("cio_array:ascii:incomplete", 6, cio_array(2341951.4, 2, x))) n++;
-  if(check("cio_array:ascii:seek", 2, cio_array(2341965.4, 2, x))) n++;
-
-  set_cio_locator_file("bad-cio-data/bad-3.txt");
-  if(check("cio_array:ascii:no-data", 1, cio_array(2341952.6, 2, x))) n++;
-
-  set_cio_locator_file("bad-cio-data/bad-4.txt");
-  if(check("cio_array:ascii:corrupt:first", -1, cio_array(2341952.6, 2, x))) n++;
-
-  set_cio_locator_file("bad-cio-data/bad-5.txt");
-  if(check("cio_array:ascii:corrupt", -1, cio_array(2341952.6, 2, x))) n++;
 
   return n;
 }
@@ -931,6 +917,7 @@ static int test_cio_location() {
 
   if(check("cio_location:ra", -1, cio_location(0.0, NOVAS_FULL_ACCURACY, NULL, &type))) n++;
   if(check("cio_location:type", -1, cio_location(0.0, NOVAS_FULL_ACCURACY, &x, NULL))) n++;
+  if(check("cio_location:accuracy", -1, cio_location(0.0, -1, &x, &type))) n++;
 
   return n;
 }
@@ -983,7 +970,8 @@ static int test_vector2radec() {
 static int test_planet_lon() {
   int n = 0;
 
-  if(check_nan("planet_lon", planet_lon(0.0, -1))) n++;
+  if(check_nan("planet_lon:-1", planet_lon(0.0, -1))) n++;
+  if(check_nan("planet_lon:pluto", planet_lon(0.0, NOVAS_PLUTO))) n++;
 
   return n;
 }
@@ -1137,7 +1125,6 @@ static int test_grav_planets() {
 }
 
 static int test_grav_undo_planets() {
-  extern int novas_inv_max_iter;
   novas_planet_bundle planets = NOVAS_PLANET_BUNDLE_INIT;
   double p[3] = {2.0}, po[3] = {0.0, 1.0}, out[3] = {0.0};
   int n = 0;
@@ -1148,10 +1135,10 @@ static int test_grav_undo_planets() {
   if(check("grav_undo_planets:pos_src", -1, grav_undo_planets(p, po, &planets, NULL))) n++;
 
   planets.mask = 1 << NOVAS_SUN;
-  novas_inv_max_iter = 0;
+  novas_set_max_iter(0);
   if(check("grav_undo_planets:converge", -1, grav_undo_planets(p, po, &planets, out))) n++;
   else if(check("grav_undo_planets:converge:errno", ECANCELED, errno)) n++;
-  novas_inv_max_iter = 100;
+  novas_set_max_iter(NOVAS_DEFAULT_MAX_ITER);
 
   return n;
 }
@@ -1166,6 +1153,9 @@ static int test_earth_sun_calc() {
   if(check("earth_sun_calc:tdb:hi", 1, earth_sun_calc(2560001.0, NOVAS_SUN, NOVAS_BARYCENTER, p, v))) n++;
   if(check("earth_sun_calc:number", 2, earth_sun_calc(NOVAS_JD_J2000, NOVAS_JUPITER, NOVAS_BARYCENTER, p, v))) n++;
 
+  if(check("earth_sun_calc:solarsystem:pos", -1, solarsystem(NOVAS_JD_J2000, NOVAS_SUN, NOVAS_BARYCENTER, NULL, v))) n++;
+  if(check("earth_sun_calc:solarsystem:OK", 0, solarsystem(NOVAS_JD_J2000, NOVAS_SUN, NOVAS_BARYCENTER, p, v))) n++;
+
   return n;
 }
 
@@ -1180,6 +1170,9 @@ static int test_earth_sun_calc_hp() {
   if(check("earth_sun_calc_hp:vel", -1, earth_sun_calc_hp(tdb2, NOVAS_SUN, NOVAS_BARYCENTER, p, NULL))) n++;
   if(check("earth_sun_calc_hp:number", 2, earth_sun_calc_hp(tdb2, NOVAS_JUPITER, NOVAS_BARYCENTER, p, v))) n++;
 
+  if(check("earth_sun_calc_hp:solarsystem_hp:tdb", -1, solarsystem_hp(NULL, NOVAS_SUN, NOVAS_BARYCENTER, p, v))) n++;
+  if(check("earth_sun_calc_hp:solarsystem_hp:OK", 0, solarsystem_hp(tdb2, NOVAS_SUN, NOVAS_BARYCENTER, p, v))) n++;
+
   enable_earth_sun_hp(0);
 
   return n;
@@ -1248,11 +1241,21 @@ static int test_time() {
   if(check_nan("time:diff_tcb:t2", novas_diff_tcb(&time, NULL))) n++;
   if(check_nan("time:diff_tcb:both", novas_diff_tcb(NULL, NULL))) n++;
 
+  return n;
+}
+
+static int test_set_str_time() {
+  int n = 0;
+
+  if(check("set_str_time:NULL", -1, novas_set_str_time(NOVAS_TT, NULL, 37, 0.11, NULL))) n++;
+  if(check("set_str_time:empty", -1, novas_set_str_time(NOVAS_TT, "", 37, 0.11, NULL))) n++;
+  if(check("set_str_time:invalid", -1, novas_set_str_time(NOVAS_TT, "blah", 37, 0.11, NULL))) n++;
 
   return n;
 }
 
 
+
 static double switching_refraction(double jd_tt, const on_surface *loc, enum novas_refraction_type type, double el) {
   static int count;
   (void) jd_tt;
@@ -1438,7 +1441,6 @@ static int test_sky_pos() {
 }
 
 static int test_app_to_geom() {
-  extern int novas_inv_max_iter;
   novas_timespec ts = NOVAS_TIMESPEC_INIT;
   observer obs = OBSERVER_INIT;
   novas_frame frame = NOVAS_FRAME_INIT;
@@ -1452,10 +1454,10 @@ static int test_app_to_geom() {
   if(check("app_to_geom:frame:init", -1, novas_app_to_geom(&frame, NOVAS_ICRS, 1.0, 2.0, 10.0, pos))) n++;
 
   novas_make_frame(NOVAS_REDUCED_ACCURACY, &obs, &ts, 0.0, 0.0, &frame);
-  novas_inv_max_iter = 0;
+  novas_set_max_iter(0);
   if(check("app_to_geom:frame:converge", -1, novas_app_to_geom(&frame, NOVAS_ICRS, 1.0, 2.0, 10.0, pos))) n++;
   else if(check("app_to_geom:frame:converge:errno", ECANCELED, errno)) n++;
-  novas_inv_max_iter = 100;
+  novas_set_max_iter(NOVAS_DEFAULT_MAX_ITER);
 
   if(check("app_to_geom:pos", -1, novas_app_to_geom(&frame, NOVAS_ICRS, 1.0, 2.0, 10.0, NULL))) n++;
   if(check("app_to_geom:sys:-1", -1, novas_app_to_geom(&frame, -1, 1.0, 2.0, 10.0, pos))) n++;
@@ -1482,6 +1484,7 @@ static int test_geom_to_app() {
   if(check("geom_to_app:frame:ok", 0, novas_geom_to_app(&frame, pos, NOVAS_ICRS, &out))) n++;
 
   if(check("geom_to_app:pos", -1, novas_geom_to_app(&frame, NULL, NOVAS_ICRS, &out))) n++;
+  if(check("geom_to_app:out", -1, novas_geom_to_app(&frame, pos, NOVAS_ICRS, NULL))) n++;
   if(check("geom_to_app:sys:-1", -1, novas_geom_to_app(&frame, pos, -1, &out))) n++;
   if(check("geom_to_app:sys:hi", -1, novas_geom_to_app(&frame, pos, NOVAS_REFERENCE_SYSTEMS, &out))) n++;
 
@@ -1696,7 +1699,7 @@ static int test_make_orbital_object() {
 static int test_orbit_posvel() {
   int n = 0;
   double pos[3] = {0.0}, vel[3] = {0.0};
-  int saved = novas_inv_max_iter;
+
   novas_orbital orbit = NOVAS_ORBIT_INIT;
 
   orbit.a = 1.0;
@@ -1708,10 +1711,10 @@ static int test_orbit_posvel() {
 
   if(check("set_obsys_pole:orbit:converge", 0, novas_orbit_posvel(0.0, &orbit, NOVAS_REDUCED_ACCURACY, pos, vel))) n++;
 
-  novas_inv_max_iter = 0;
+  novas_set_max_iter(0);
   if(check("set_obsys_pole:orbit:converge", -1, novas_orbit_posvel(0.0, &orbit, NOVAS_REDUCED_ACCURACY, pos, vel))) n++;
   else if(check("set_obsys_pole:orbit:converge:errno", ECANCELED, errno)) n++;
-  novas_inv_max_iter = saved;
+  novas_set_max_iter(NOVAS_DEFAULT_MAX_ITER);
 
   orbit.system.type = -1;
   if(check("set_obsys_pole:orbit:type:-1", -1, novas_orbit_posvel(0.0, &orbit, NOVAS_REDUCED_ACCURACY, pos, vel))) n++;
@@ -1868,7 +1871,6 @@ static int test_rise_set() {
   novas_timespec time = NOVAS_TIMESPEC_INIT;
   observer obs = OBSERVER_INIT;
   novas_frame frame = NOVAS_FRAME_INIT;
-  int save = novas_inv_max_iter;
 
   if(check_nan("rise_set:rises_above:frame:null", novas_rises_above(0.0, &sun, NULL, NULL))) n++;
   if(check_nan("rise_set:rises_above:frame:init", novas_rises_above(0.0, &sun, &frame, NULL))) n++;
@@ -1891,10 +1893,10 @@ static int test_rise_set() {
   if(check_nan("rise_set:rises_above:source:null", novas_rises_above(31.0, &sun, &frame, NULL))) n++;
   if(check_nan("rise_set:sets_below:source:null", novas_sets_below(31.0, &sun, &frame, NULL))) n++;
 
-  novas_inv_max_iter = 0;
+  novas_set_max_iter(0);
   if(check_nan("rise_set:rises_above:noconv", novas_rises_above(0.0, &sun, &frame, NULL))) n++;
   if(check_nan("rise_set:sets_below:noconv", novas_rises_above(0.0, &sun, &frame, NULL))) n++;
-  novas_inv_max_iter = save;
+  novas_set_max_iter(NOVAS_DEFAULT_MAX_ITER);
 
   return n;
 }
@@ -2290,10 +2292,9 @@ static int test_approx_sky_pos() {
 static int test_moon_phase() {
   int n = 0;
 
-  int saved = novas_inv_max_iter;
-  novas_inv_max_iter = 0;
+  novas_set_max_iter(0);
   if(check_nan("moon_phase:conv", novas_moon_phase(NOVAS_JD_J2000))) n++;
-  novas_inv_max_iter = saved;
+  novas_set_max_iter(NOVAS_DEFAULT_MAX_ITER);
   return n;
 }
 
@@ -2303,15 +2304,192 @@ static int test_next_moon_phase() {
   if(check_nan("next_moon_phase:time:lo", novas_next_moon_phase(0.0, NOVAS_JD_J2000 - 31.0 * JULIAN_CENTURY_DAYS))) n++;
   if(check_nan("next_moon_phase:time:hi", novas_next_moon_phase(0.0, NOVAS_JD_J2000 + 31.0 * JULIAN_CENTURY_DAYS))) n++;
 
-  int saved = novas_inv_max_iter;
-  novas_inv_max_iter = 0;
+  novas_set_max_iter(0);
   if(check_nan("next_moon_phase:conv", novas_next_moon_phase(0.0, NOVAS_JD_J2000))) n++;
-  novas_inv_max_iter = saved;
+  novas_set_max_iter(NOVAS_DEFAULT_MAX_ITER);
   return n;
 }
 
-int main() {
+static int test_libration() {
   int n = 0;
+  double xp, yp, dut;
+
+  if(check("libration:args", -1, novas_diurnal_libration(0.0, NULL, &xp, &yp, &dut))) n++;
+
+  return n;
+}
+
+static int test_ocean_tides() {
+  int n = 0;
+  double xp, yp, dut;
+
+  if(check("ocean_tides:args", -1, novas_diurnal_ocean_tides(0.0, NULL, &xp, &yp, &dut))) n++;
+
+  return n;
+}
+
+static int test_diurnal_eop() {
+  int n = 0;
+  double xp, yp, dut;
+
+  if(check("diurnal_eop:args", -1, novas_diurnal_eop(0.0, NULL, &xp, &yp, &dut))) n++;
+
+  return n;
+}
+
+static int test_diurnal_eop_at_time() {
+  int n = 0;
+  double xp, yp, dut;
+
+  if(check("diurnal_eop_at_time:time", -1, novas_diurnal_eop_at_time(NULL, &xp, &yp, &dut))) n++;
+
+  return n;
+}
+
+static int test_cartesian_to_geodetic() {
+  int n = 0;
+  double x[3] = {0.0};
+  double lon, lat, alt;
+
+  if(check("cartesian_to_geodetic:x", -1, novas_cartesian_to_geodetic(NULL, NOVAS_GRS80_ELLIPSOID, &lon, &lat, &alt))) n++;
+  if(check("cartesian_to_geodetic:ellipsoid", -1, novas_cartesian_to_geodetic(x, -1, &lon, &lat, &alt))) n++;
+
+  return n;
+}
+
+static int test_geodetic_to_cartesian() {
+  int n = 0;
+  double x[3] = {0.0};
+
+  if(check("geodetic_to_cartesian:x", -1, novas_geodetic_to_cartesian(0.0, 0.0, 0.0, NOVAS_GRS80_ELLIPSOID, NULL))) n++;
+  if(check("geodetic_to_cartesian:ellipsoid", -1, novas_geodetic_to_cartesian(0.0, 0.0, 0.0, -1, x))) n++;
+
+  return n;
+}
+
+static int test_itrf_transform() {
+  int n = 0;
+  double from_coords[3] = {0.0}, from_rates[3] = {0.0}, to_coords[3] = {0.0}, to_rates[3] = {0.0};
+
+  if(check("itrf_transform:from_coords", -1, novas_itrf_transform(2000, NULL, from_rates, 2014, to_coords, to_rates))) n++;
+  if(check("itrf_transform:rates", -1, novas_itrf_transform(2000, from_coords, NULL, 2014, to_coords, to_rates))) n++;
+
+  return n;
+}
+
+static int test_clock_skew() {
+  int n = 0;
+  observer obs = {};
+  novas_timespec time = {};
+  novas_frame frame = {};
+
+  if(check_nan("clock_skew:frame", novas_mean_clock_skew(NULL, NOVAS_TCG))) n++;
+  if(check_nan("clock_skew:frame:init", novas_mean_clock_skew(&frame, NOVAS_TCG))) n++;
+
+  make_observer_at_geocenter(&obs);
+  novas_set_time(NOVAS_TT, NOVAS_JD_J2000, 32.0, 0.0, &time);
+
+  enable_earth_sun_hp(1);
+  novas_make_frame(NOVAS_FULL_ACCURACY, &obs, &time, 0.0, 0.0, &frame);
+
+  if(check_nan("clock_skew:timescale:-1", novas_mean_clock_skew(&frame, -1))) n++;
+  if(check_nan("clock_skew:timescale:UT1", novas_mean_clock_skew(&frame, NOVAS_UT1))) n++;
+
+  if(check_nan("clock_skew:hp:no_planets", novas_mean_clock_skew(&frame, NOVAS_TCG))) n++;
+  enable_earth_sun_hp(0);
+
+  return n;
+}
+
+static int test_cat_entry() {
+  int n = 0;
+  char *name = "blah", longname[SIZE_OF_OBJ_NAME + 2] = {};
+  cat_entry star = {};
+
+  if(check("cat_entry:init", -1, novas_init_cat_entry(NULL, name, 0.0, 0.0))) n++;
+
+  memset(longname, 'x', SIZE_OF_OBJ_NAME);
+  if(check("cat_entry:init:long", -1, novas_init_cat_entry(NULL, longname, 0.0, 0.0))) n++;
+
+  longname[SIZE_OF_OBJ_NAME + 1] = 'x';
+  if(check("cat_entry:init:verylong", -1, novas_init_cat_entry(NULL, longname, 0.0, 0.0))) n++;
+
+  if(check("cat_entry:set_catalog", -1, novas_set_catalog(NULL, name, 0))) n++;
+  if(check("cat_entry:set_proper_motion", -1, novas_set_proper_motion(NULL, 0.0, 0.0))) n++;
+  if(check("cat_entry:set_parallax", -1, novas_set_parallax(NULL, 0.0))) n++;
+  if(check("cat_entry:set_distance", -1, novas_set_distance(NULL, 0.0))) n++;
+  if(check("cat_entry:set_ssb_vel", -1, novas_set_ssb_vel(NULL, 0.0))) n++;
+  if(check("cat_entry:set_ssb_vel:hi", -1, novas_set_ssb_vel(&star, NOVAS_C + 1.0))) n++;
+  if(check("cat_entry:set_ssb_vel:lo", -1, novas_set_ssb_vel(&star, -(NOVAS_C + 1.0)))) n++;
+  if(check("cat_entry:set_lsr_vel", -1, novas_set_lsr_vel(NULL, 2000.0, 0.0))) n++;
+  if(check("cat_entry:set_redshift", -1, novas_set_redshift(NULL, 0.0))) n++;
+
+  return n;
+}
+
+static int test_make_itrf_site() {
+  int n = 0;
+  on_surface site;
+
+  if(check("make_itrf_site:site", -1, make_itrf_site(0.0, 0.0, 0.0, NULL))) n++;
+  if(check("make_itrf_site:lat:-91", -1, make_itrf_site(-91.0, 0.0, 0.0, &site))) n++;
+  if(check("make_itrf_site:lat:+91", -1, make_itrf_site(91.0, 0.0, 0.0, &site))) n++;
+
+  return n;
+}
+
+static int test_make_xyz_site() {
+  int n = 0;
+  on_surface site;
+  double xyz[3] = {0.0};
+
+  if(check("make_xyz_site:xyz", -1, make_xyz_site(NULL, &site))) n++;
+  if(check("make_xyz_site:site", -1, make_xyz_site(xyz, NULL))) n++;
+
+  return n;
+}
+
+static int test_make_observer_at_site() {
+  int n = 0;
+  observer obs;
+  on_surface site;
+
+  if(check("make_observer_at_site:site", -1, make_observer_at_site(NULL, &obs))) n++;
+  if(check("make_observer_at_site:obs", -1, make_observer_at_site(&site, NULL))) n++;
+
+  return n;
+}
+
+static int test_set_default_weather() {
+  if(check("set_default_weather", -1, novas_set_default_weather(NULL))) return 1;
+  return 0;
+}
+
+static int test_itrf_transform_site() {
+  int n = 0;
+  on_surface site = {};
+
+  if(check("itrf_transform_site:in", -1, novas_itrf_transform_site(2003, NULL, 2014, &site))) n++;
+  if(check("itrf_transform_site:out", -1, novas_itrf_transform_site(2003, &site, 2014, NULL))) n++;
+
+  return n;
+}
+
+static int test_transform_geodetic() {
+  int n = 0;
+  on_surface site = {};
+
+  if(check("transform_geodetic:in", -1, novas_geodetic_transform_site(NOVAS_WGS84_ELLIPSOID, NULL, NOVAS_GRS80_ELLIPSOID, &site))) n++;
+  if(check("transform_geodetic:out", -1, novas_geodetic_transform_site(NOVAS_WGS84_ELLIPSOID, &site, NOVAS_GRS80_ELLIPSOID, NULL))) n++;
+
+  return n;
+}
+
+int main(int argc, const char *argv[]) {
+  int n = 0;
+
+  if(argc > 1)
+    dataPath = (char *) argv[1];
 
   if(test_v2z()) n++;
   if(test_z2v()) n++;
@@ -2344,6 +2522,8 @@ int main() {
   if(test_tod_to_j2000()) n++;
   if(test_gcrs_to_cirs()) n++;
   if(test_cirs_to_gcrs()) n++;
+  if(test_cirs_to_tod()) n++;
+  if(test_tod_to_cirs()) n++;
   if(test_cirs_to_app_ra()) n++;
   if(test_app_to_cirs_ra()) n++;
 
@@ -2498,6 +2678,25 @@ int main() {
   if(test_moon_phase()) n++;
   if(test_next_moon_phase()) n++;
 
+  if(test_libration()) n++;
+  if(test_ocean_tides()) n++;
+  if(test_diurnal_eop()) n++;
+  if(test_diurnal_eop_at_time()) n++;
+
+  if(test_cartesian_to_geodetic()) n++;
+  if(test_geodetic_to_cartesian()) n++;
+  if(test_itrf_transform()) n++;
+
+  if(test_clock_skew()) n++;
+  if(test_cat_entry()) n++;
+  if(test_set_str_time()) n++;
+  if(test_make_itrf_site()) n++;
+  if(test_make_xyz_site()) n++;
+  if(test_make_observer_at_site()) n++;
+  if(test_set_default_weather()) n++;
+  if(test_itrf_transform_site()) n++;
+  if(test_transform_geodetic()) n++;
+
   if(n) fprintf(stderr, " -- FAILED %d tests\n", n);
   else fprintf(stderr, " -- OK\n");
 
